/*
	File: mixdp.h (core)

	Copyright (c) 1994, 1995, 1996 Macromedia, Inc.  All Rights Reserved

	You may utilize this source file to create and compile object code for use within products
	you may create.  THIS CODE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, AND
	MACROMEDIA DISCLAIMS ALL IMPLIED WARRANTIES INCLUDING, BUT NOT LIMITED TO, MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT WILL MACROMEDIA BE
	LIABLE TO YOU FOR ANY CONSEQUENTIAL, INDIRECT OR INCIDENTAL DAMAGES ARISING OUT OF YOUR
	USE OR INABILITY TO USE THIS CODE.

	Abstract:

  	Revision history:
	951018	Original. (srj)
*/

#ifndef _H_mixdp
#define _H_mixdp

#ifndef _H_moastdif
	#include "moastdif.h"
#endif

#ifndef moa_in
	#define moa_in
#endif
#ifndef moa_inout
	#define moa_inout
#endif
#ifndef moa_out
	#define moa_out
#endif
#ifndef MAKE_MOASTATUS
	#define MAKE_MOASTATUS(code) ((MoaError)((MoaUlong)(0L << 31) | (MoaUlong)(4L << 16) | (MoaUlong)(code)))
#endif
#ifndef PUREMETHOD
	#define PUREMETHOD(macro_NAME, macro_ARGS) \
		STDMETHOD(macro_NAME) macro_ARGS PURE;
#endif
#ifndef PUREMETHOD_
	#define PUREMETHOD_(macro_RET, macro_NAME, macro_ARGS) \
		STDMETHOD_(macro_RET, macro_NAME) macro_ARGS PURE;
#endif
#ifndef EXTERN_DEFMETHOD
	#define EXTERN_DEFMETHOD(macro_NAME, macro_ARGS) \
		EXTERN_DEFINE_METHOD(MoaError, macro_NAME, macro_ARGS)
#endif
#ifndef EXTERN_DEFMETHOD_
	#define EXTERN_DEFMETHOD_(macro_RET, macro_NAME, macro_ARGS) \
		EXTERN_DEFINE_METHOD(macro_RET, macro_NAME, macro_ARGS)
#endif

#if defined(__MWERKS__) && defined(__MC68K__)
#pragma push
#pragma pointers_in_D0
#endif

#ifdef DATAPIPE

/* -------------------------------------------------------------------------- */
/* ------------------------ DataPipe Definitions -------------------------- */
/* -------------------------------------------------------------------------- */

/* -------- Error Codes ----- */
/*
	returned by GetDpData meaning "data hasn't been returned yet."
*/
#define kMoaDpStatus_TryAgainLater		kMoaMixStatus_TryAgainLater

/*
	returned by GetDpData along with a DataObject, meaning,
	"here's an INTERMEDIATE update to the data." The caller
	should continue polling until kMoaErr_NoErr is returned,
	however. Note: this should only be returned if the
	key DataPipeAllowIU was TRUE in the request.
*/
#define kMoaDpStatus_IntermediateData	kMoaMixStatus_IntermediateData

/*
	returned by RequestDpData meaning "I can't provide
	this kind of data"
*/
#define kMoaDpErr_WrongType				MAKE_MOAERR(4902)

/*
	You can only call RequestDpData once per pipe.
	Create a new pipe to make a new request.

	RequestDpData returns this if you call it multiple times.
	GetDpData returns this if you call it before RequestDpData.
*/
#define kMoaDpErr_OnlyOneRequest		kMoaMixErr_WrongCallSequence

/*
	data transmission error.
*/
#define kMoaDpErr_BadPacket				MAKE_MOAERR(4904)

/*
	returned by RequestDpData or GetDpData meaning "I'm never
	going to provide you this data; user won't let me"
	(same as kMoaMixErr_UserCanceled)
*/
#define kMoaDpErr_Canceled				MAKE_MOAERR(21000)

/* ----------- DataPipe data types ---------- */
//??? hack
#if defined(WINDOWS) || defined(_WINDOWS)
	typedef MoaID						MoaPipeServerID;
#else
	#ifndef __PROCESSES__
		#include <Processes.h>
	#endif
	typedef ProcessSerialNumber 		MoaPipeServerID;
#endif
typedef MoaPipeServerID FAR * 			PMoaPipeServerID;
typedef const MoaPipeServerID FAR * 	ConstPMoaPipeServerID;

#undef INTERFACE
#define INTERFACE IMoaDataPipe
DECLARE_INTERFACE_(IMoaDataPipe, IMoaUnknown)
{
	STD_IUNKNOWN_METHODS

	PUREMETHOD(RequestDpData, (THIS_
		moa_in PIMoaDict pRequestDict,
		moa_in PIMoaDataObject pRequestSourceData	/* could be NULL */
	))

	PUREMETHOD(GetDpData, (THIS_
		moa_out PIMoaDataObject FAR * ppDataObject
	))
};
typedef IMoaDataPipe FAR * PIMoaDataPipe;
DEFINE_GUID(IID_IMoaDataPipe, 0xED7EC0F4L, 0x33C0, 0x11CF, 0xB9, 0x6E, 0x08, 0x00, 0x07, 0x16, 0x0D, 0xC3);

	/* -------------------------------------------------------------------------- */

	/* the dictionary passed to RequestDpData may contain
		the following information. */

	/* this is the format of data requested (a clipboard format). 
		the server should respond with (at least) this data
		(it can provide other formats too if it wants to) */
	#define kMoaDictKey_DataPipeRequestDpDataFormat		"DataFormat"
	#define kMoaDictType_DataPipeRequestDpDataFormat	kMoaDictType_CString

	/* if TRUE, the user must be allowed to edit the data before
		the data is provided. */
	#define kMoaDictKey_DataPipeRequireEdit		"RequireEdit"
	#define kMoaDictType_DataPipeRequireEdit	kMoaDictType_Bool

	/* if TRUE, the user must NOT be allowed to edit the data before
		the data is provided. */
	#define kMoaDictKey_DataPipeForbidEdit		"ForbidEdit"
	#define kMoaDictType_DataPipeForbidEdit		kMoaDictType_Bool

	/* if TRUE, intermediate updates to the data are allowed.
		if this key is false or not present, intermediate updates
		(via GetDpData) should not be done by the server. */
	#define kMoaDictKey_DataPipeAllowIU			"AllowIU"
	#define kMoaDictType_DataPipeAllowIU		kMoaDictType_Bool

	/* additional use-specific keys possible as well! */

#endif

#endif

