/*
 *	FILE:		xh_cache.cpp
 *	PRODUCT:	ALL
 *	SYNOPSIS:	Implementation of helper routines associated with IMoaCache.
 *	
 *	Copyright (c) 1994, 1995, 1996 Macromedia, Inc.  All Rights Reserved.
 *
 *	You may utilize this source file to create and compile object code for use 
 *	within products you may create.  THIS CODE IS PROVIDED "AS IS", WITHOUT 
 *	WARRANTY OF ANY KIND, AND MACROMEDIA DISCLAIMS ALL IMPLIED WARRANTIES 
 *	INCLUDING, BUT NOT LIMITED TO, MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 *	PURPOSE AND NONINFRINGEMENT.  IN NO EVENT WILL MACROMEDIA BE LIABLE TO YOU 
 *	FOR ANY CONSEQUENTIAL, INDIRECT OR INCIDENTAL DAMAGES ARISING OUT OF YOUR
 *	USE OR INABILITY TO USE THIS CODE.
 */


/*
 *	include files
 */
#include "moaxtra.h"
#include "xh_cache.h"
#include "moatry.h"


/*
 *	special (required for 68k)
 */
#if defined(__MWERKS__) && !defined(powerc)
#pragma push
#pragma pointers_in_D0
#endif


/*
 *	type declarations
 */

typedef struct {
	ConstPMoaClassID		pClassID;
	ConstPMoaInterfaceID	pInterfaceID;
	PIMoaRegistryEntryDict	pRegistryDict;
	MoaBoolParam			bUnique;
} RegistryEntryData, * PRegistryEntryData;


/*
 *	internal prototypes
 */

STDMETHODIMP_(MoaError) _RegistryEntryEnumProc(
	PIMoaRegistryEntryDict pRegistryDict,
	ConstPMoaClassID pClassID,
	ConstPMoaInterfaceID pInterfaceID,
	PMoaVoid refCon
);


/*
 *	internal routines
 */


/* ------------------------------------------------------ _RegistryEntryEnumProc
 *
 *	This routine is called by IMoaCache::EnumerateRegistryEntries() for every
 *	registry entry. The refCon is assumed to point to a RegistryEntryData
 *	struct. When it is given a CLSID and an IID that matches those
 *	in the refCon, it assigns the given PIMoaRegistryEntryDict to the
 *	appropriate entry in the refCon.
 *
 *	ARGUMENTS
 *	---------
 *	pRegistryDict	-->		the dictionary associated with the CLSID/IID pair
 *	pClassID		-->		the id of the class
 *	pInterfaceID	-->		the id of the interface
 *	refCon			<->		the RegistryEntryData structure
 *
 *	RETURN TYPE
 *	-----------
 *	MoaError
 */
STDMETHODIMP_(MoaError) _RegistryEntryEnumProc(
	PIMoaRegistryEntryDict pRegistryDict,
	ConstPMoaClassID pClassID,
	ConstPMoaInterfaceID pInterfaceID,
	PMoaVoid refCon
)
{
moa_try

	/* variable declarations */
	PRegistryEntryData		pData;
	
	/* variable initialization */
	pData = (PRegistryEntryData) refCon;
	
	/* test for a match */
	if (MoaEqualID( pData->pClassID, pClassID ) &&
		MoaEqualID( pData->pInterfaceID, pInterfaceID ) ) {

		if (pData->pRegistryDict) {
			/* uh oh... more than one with the same CLSID/IID pair */
			pData->bUnique = FALSE;
			Throw( 1 );
		}
		pData->pRegistryDict = pRegistryDict;
	}		

moa_catch

moa_catch_end

moa_try_end
}


/*
 *	external routines
 */
 
/* -------------------------------------------------------- GetRegistryEntryDict
 *
 *	Uses the IMoaCache::EnumerateRegistryEntries() method to retrieve a 
 *	PIMoaRegistryEntryDict given a CLSID and an IID.
 *
 *	THIS HELPER SHOULD BE USED WITH CAUTION! For example, Director Xtras
 *	commonly register the same CLSID/IID pair multiple times. In such cases
 *	this helper is NOT useful.
 *
 *	ARGUMENTS
 *	---------
 *	pCallback		-->		a pointer to the IMoaCallback object
 *	pClassID		-->		the id of the class
 *	pInterfaceID	-->		the id of the interface
 *	pRegistryDict	<--		the dictionary associated with the CLSID/IID pair
 *
 *	RETURN TYPE
 *	-----------
 *	MoaBoolParam			FALSE if there is more than one dictionary
 *							associated with the given CLSID/IID pair or if
 *							the enumeration simply failed.
 */
MoaBoolParam GetRegistryEntryDict(
	PIMoaCallback pCallback,
	ConstPMoaClassID pClassID,
	ConstPMoaInterfaceID pInterfaceID,
	PIMoaRegistryEntryDict * ppRegistryDict
)
{
moa_try

	/* variable declarations */
	PIMoaCache			pCache;
	RegistryEntryData	theData;
	MoaBoolParam		returnVal;
	
	/* variable initialization */
	pCache = pCallback->MoaGetCache();
	theData.pClassID = pClassID;
	theData.pInterfaceID = pInterfaceID;
	theData.pRegistryDict = NULL;
	theData.bUnique = TRUE;
	returnVal = TRUE;
	
	/* enumerate the entries */
	ThrowErr( pCache->EnumerateRegistryEntries((MoaCacheRegistryEntryEnumProc)_RegistryEntryEnumProc, &theData) );
	ThrowFailed( theData.bUnique );
	*ppRegistryDict = theData.pRegistryDict;

moa_catch

	/* report the exception */
	MOA_DEBUGSTR( ("Exception thrown at %s.", moa_get_exception_location) );
	returnVal = FALSE;

moa_catch_end

	/* release the cache */
	pCache->Release();

moa_try_end_( MoaBoolParam, returnVal )
}


/*
 *	special (required for 68k)
 */
#if defined(__MWERKS__) && !defined(powerc)
#pragma pop
#endif
