/*
 *	FILE:		xh_path.cpp
 *	PRODUCT:	FreeHand
 *	SYNOPSIS:	Implementation of helper routines associated with FreeHand paths.
 *	
 *	Copyright (c) 1994, 1995, 1996 Macromedia, Inc.  All Rights Reserved.
 *
 *	You may utilize this source file to create and compile object code for use 
 *	within products you may create.  THIS CODE IS PROVIDED "AS IS", WITHOUT 
 *	WARRANTY OF ANY KIND, AND MACROMEDIA DISCLAIMS ALL IMPLIED WARRANTIES 
 *	INCLUDING, BUT NOT LIMITED TO, MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 *	PURPOSE AND NONINFRINGEMENT.  IN NO EVENT WILL MACROMEDIA BE LIABLE TO YOU 
 *	FOR ANY CONSEQUENTIAL, INDIRECT OR INCIDENTAL DAMAGES ARISING OUT OF YOUR
 *	USE OR INABILITY TO USE THIS CODE.
 */


/*
 *	include files
 */
#include "moaxtra.h"
#include "fhxif.h"
#include "xh_path.h"
#include "moatry.h"


/*
 *	special (required for 68k)
 */
#if defined(__MWERKS__) && !defined(powerc)
#pragma push
#pragma pointers_in_D0
#endif


/*
 *	internal prototypes
 */
 
MoaBoolParam _IsValidPointIndex(
	MoaLong pointIndex,
	MoaLong numPoints
);


/*
 *	internal routines
 */

/* ----------------------------------------------------------- _IsValidPointIndex
 *
 *	Determines if a point index is valid given the total number of points.
 *
 *	ARGUMENTS
 *	---------
 *	pointIndex		-->		point index to test
 *	numPoints		-->		number of points on the path
 *
 *	RETURN TYPE
 *	-----------
 *	MoaBoolParam			TRUE if point index is valid, FALSE otherwise
 */
MoaBoolParam _IsValidPointIndex(
	MoaLong pointIndex,
	MoaLong numPoints
)
{
moa_try

	/* variable declarations */
	MoaBoolParam 		isValid;
	
	isValid = (	(pointIndex >= 0) &&
				(pointIndex < numPoints) ) ? TRUE : FALSE;

moa_catch

moa_catch_end

moa_try_end_( MoaBoolParam, isValid )
}


/*
 *	external routines
 */
 

/* ----------------------------------------------------------------- GetPathInfo
 *
 *	Retrieves information about a given path.
 *
 *	ARGUMENTS
 *	---------
 *	pFhCbk			-->		pointer to the IMoaFhCallback object
 *	pElem			-->		the element to examine
 *	pPathInfo		<--		returns information about the path
 *
 *	RETURN TYPE
 *	-----------
 *	MoaBoolParam			TRUE if successful, FALSE otherwise.
 */
MoaBoolParam GetPathInfo(
	PIMoaFhCallback pFhCbk,
	ElemparamP pElem,
	PFhPathInfoRec pPathInfo
)
{
moa_try

	/* variable declarations */
	MoaBoolParam		result;
	
	/* get path information */
	if (pFhCbk->FHIElemType(pElem) == ELEM_TYPE_PATH) {
		pPathInfo->pElem = pElem;
		pPathInfo->numPoints = pFhCbk->FHIPathGetNumTriples(pElem);
		pPathInfo->isClosed = pFhCbk->FHIPathIsClosed(pElem);
		result = TRUE;
	}
	else {
		pPathInfo->pElem = NULL;
		pPathInfo->numPoints = 0;
		pPathInfo->isClosed = FALSE;
		result = FALSE;
	}

moa_catch

	/* report the exception */
	MOA_DEBUGSTR( ("Exception thrown at %s.", moa_get_exception_location) );
	result = FALSE;

moa_catch_end

moa_try_end_( MoaBoolParam, result )
}


/* ----------------------------------------------------------------- GetFhTriple
 *
 *	Retrieves the location and flags of a triple on a given path.
 *
 *	ARGUMENTS
 *	---------
 *	pFhCbk			-->		pointer to the IMoaFhCallback object
 *	pPathInfo		-->		information about the path
 *	pointIndex		-->		the index of the triple to retrieve
 *	pTriple			<--		returns location and flags of the triple
 *
 *	RETURN TYPE
 *	-----------
 *	MoaBoolParam			TRUE if successful, FALSE otherwise.
 */
MoaBoolParam GetFhTriple(
	PIMoaFhCallback pFhCbk,
	PFhPathInfoRec pPathInfo,
	MoaLong pointIndex,
	PFhTripleRec pTriple
)
{
moa_try

	/* variable declarations */
	MoaBoolParam		result;
	
	/* validate */
	ThrowNull( pPathInfo->pElem );
	ThrowNull( _IsValidPointIndex(pointIndex, pPathInfo->numPoints) );
	
	/* get information about the triple */
	ThrowNull( pFhCbk->FHIPathGetTriple(pPathInfo->pElem, pointIndex, &pTriple->predBCP, &pTriple->mainPt, &pTriple->succBCP) );
	pTriple->tripleFlags = pFhCbk->FHIPathGetTripleFlags(pPathInfo->pElem, pointIndex);
	result = TRUE;
	

moa_catch

	result = FALSE;

moa_catch_end

moa_try_end_( MoaBoolParam, result )
}


/* ----------------------------------------------------------------- SetFhTriple
 *
 *	Sets the location and flags of a triple on a given path.
 *
 *	ARGUMENTS
 *	---------
 *	pFhCbk			-->		pointer to the IMoaFhCallback object
 *	pPathInfo		-->		information about the path
 *	pointIndex		-->		the index of the triple to set
 *	pTriple			-->		location and flags of the triple
 *
 *	RETURN TYPE
 *	-----------
 *	MoaBoolParam			TRUE if successful, FALSE otherwise.
 */
MoaBoolParam SetFhTriple(
	PIMoaFhCallback pFhCbk,
	PFhPathInfoRec pPathInfo,
	MoaLong pointIndex,
	PFhTripleRec pTriple
)
{
moa_try

	/* variable declarations */
	MoaBoolParam		result;
	
	/* validate */
	ThrowNull( pPathInfo->pElem );
	ThrowNull( _IsValidPointIndex(pointIndex, pPathInfo->numPoints) );
	
	/* get information about the triple */
	ThrowNull( pFhCbk->FHIPathSetTriple(pPathInfo->pElem, pointIndex, &pTriple->predBCP, &pTriple->mainPt, &pTriple->succBCP) );
	pFhCbk->FHIPathSetTripleFlags(pPathInfo->pElem, pointIndex, pTriple->tripleFlags);
	result = TRUE;
	

moa_catch

	result = FALSE;

moa_catch_end

moa_try_end_( MoaBoolParam, result )
}


/*
 *	special (required for 68k)
 */
#if defined(__MWERKS__) && !defined(powerc)
#pragma pop
#endif
