/*
 *	FILE:		xh_track.cpp
 *	PRODUCT:	FreeHand
 *	SYNOPSIS:	Implementation of helper routines associated with tool tracking.
 *	
 *	Copyright (c) 1994, 1995, 1996 Macromedia, Inc.  All Rights Reserved.
 *
 *	You may utilize this source file to create and compile object code for use 
 *	within products you may create.  THIS CODE IS PROVIDED "AS IS", WITHOUT 
 *	WARRANTY OF ANY KIND, AND MACROMEDIA DISCLAIMS ALL IMPLIED WARRANTIES 
 *	INCLUDING, BUT NOT LIMITED TO, MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 *	PURPOSE AND NONINFRINGEMENT.  IN NO EVENT WILL MACROMEDIA BE LIABLE TO YOU 
 *	FOR ANY CONSEQUENTIAL, INDIRECT OR INCIDENTAL DAMAGES ARISING OUT OF YOUR
 *	USE OR INABILITY TO USE THIS CODE.
 */


/*
 *	include files
 */
#include "moaxtra.h"
#include "fhxif.h"
#include "xh_track.h"
#include "moatry.h"


/*
 *	special (required for 68k)
 */
#if defined(__MWERKS__) && !defined(powerc)
#pragma push
#pragma pointers_in_D0
#endif


/*
 *	internal prototypes
 */
 
void _FeedbackMoveProc(
	ConstPMPoint pPoint,
	PMoaVoid pData
);

void _FeedbackLineProc(
	ConstPMPoint pPoint,
	PMoaVoid pData
);

void _FeedbackCurveProc(
	ConstPMPoint pSuccBCP,
	ConstPMPoint pPredBCP,
	ConstPMPoint pNextPt,
	PMoaVoid pData
);

void _FeedbackCloseProc(
	PMoaVoid pData
);


/*
 *	internal routines
 */

/* ----------------------------------------------------------- _FeedbackMoveProc
 *
 *	Handles all "move to" operations in the enumerated path.
 *
 *	ARGUMENTS
 *	---------
 *	pPoint			-->		the point to move to
 *	pData			-->		pointer to the IMoaFhCallback object
 *
 *	RETURN TYPE
 *	-----------
 *	void
 */
void _FeedbackMoveProc(
	ConstPMPoint pPoint,
	PMoaVoid pData
)
{
moa_try

	/* variable declarations */
	PIMoaFhCallback		pFhCbk;
	
	/* variable initialization */
	pFhCbk = (PIMoaFhCallback) pData;
	
	/* perform the "move to" op */
	pFhCbk->FHITKMoveTo(pPoint);

moa_catch

moa_catch_end

moa_try_end_void
}


/* ----------------------------------------------------------- _FeedbackLineProc
 *
 *	Handles all "line to" operations in the enumerated path.
 *
 *	ARGUMENTS
 *	---------
 *	pPoint			-->		the point to draw a line to
 *	pData			-->		pointer to the IMoaFhCallback object
 *
 *	RETURN TYPE
 *	-----------
 *	void
 */
void _FeedbackLineProc(
	ConstPMPoint pPoint,
	PMoaVoid pData
)
{
moa_try

	/* variable declarations */
	PIMoaFhCallback		pFhCbk;
	
	/* variable initialization */
	pFhCbk = (PIMoaFhCallback) pData;
	
	/* perform the "move to" op */
	pFhCbk->FHITKLineTo(pPoint);

moa_catch

moa_catch_end

moa_try_end_void
}


/* ---------------------------------------------------------- _FeedbackCurveProc
 *
 *	Handles all "curve to" operations in the enumerated path.
 *
 *	ARGUMENTS
 *	---------
 *	pPoint			-->		the point to draw a curve to
 *	pData			-->		pointer to the IMoaFhCallback object
 *
 *	RETURN TYPE
 *	-----------
 *	void
 */
void _FeedbackCurveProc(
	ConstPMPoint pSuccBCP,
	ConstPMPoint pPredBCP,
	ConstPMPoint pNextPt,
	PMoaVoid pData
)
{
moa_try

	/* variable declarations */
	PIMoaFhCallback		pFhCbk;
	
	/* variable initialization */
	pFhCbk = (PIMoaFhCallback) pData;
	
	/* perform the "move to" op */
	pFhCbk->FHITKCurveTo(pSuccBCP, pPredBCP, pNextPt);

moa_catch

moa_catch_end

moa_try_end_void
}


/* ---------------------------------------------------------- _FeedbackCloseProc
 *
 *	Handles all "close" operations in the enumerated path.
 *
 *	ARGUMENTS
 *	---------
 *	pData			-->		pointer to the IMoaFhCallback object
 *
 *	RETURN TYPE
 *	-----------
 *	void
 */
void _FeedbackCloseProc(
	PMoaVoid pData
)
{
moa_try

	/* variable declarations */
	PIMoaFhCallback		pFhCbk;
	
	/* variable initialization */
	pFhCbk = (PIMoaFhCallback) pData;
	
	/* perform the "close" op */
	pFhCbk->FHITKClosePath();

moa_catch

moa_catch_end

moa_try_end_void
}


/*
 *	external routines
 */
 
/* -------------------------------------------------------- TrackFeedbackElement
 *
 *	Traces a path element during tool tracking feedback. A tool must be active;
 *	in other words, this routine should be called from 
 *	IMoaFhToolItem_TrackFeedback(). Note also that this routine does not call
 *	FHITKHiliteStroke(), which actually draws all FHITK...() calls on the
 *	pasteboard.
 *
 *	ARGUMENTS
 *	---------
 *	pFhCbk			-->		a pointer to the IMoaFhCallback object
 *	pElem			-->		the element to trace
 *
 *	RETURN TYPE
 *	-----------
 *	MoaBoolParam			TRUE if successful, FALSE otherwise.
 */
MoaBoolParam TrackFeedbackElement(
	PIMoaFhCallback pFhCbk,
	ElemparamP pElem
)
{
	return TrackFeedbackElementMatrix(pFhCbk, pElem, NULL);
}

/* -------------------------------------------------- TrackFeedbackElementMatrix
 *
 *	This routine is similar to TrackFeedbackElement except that it allows you 
 *  to translate the feedback draw with a matrix before that feedback is 
 *  displayed.  
 *
 *  The same warnings that apply to TrackFeedbackElement apply here as well;
 *  Namely:
 *	A tool must be active; in other words, this routine should be called from 
 *	IMoaFhToolItem_TrackFeedback(). Note also that this routine does not call
 *	FHITKHiliteStroke(), which actually draws all FHITK...() calls on the
 *	pasteboard.
 *
 *	ARGUMENTS
 *	---------
 *	pFhCbk			-->		a pointer to the IMoaFhCallback object
 *	pElem			-->		the element to trace
 *	pMatrix			-->		a matrix that is used to transform the element's feedback
 *
 *	RETURN TYPE
 *	-----------
 *	MoaBoolParam			TRUE if successful, FALSE otherwise.
 */
MoaBoolParam TrackFeedbackElementMatrix(
	PIMoaFhCallback pFhCbk,
	ElemparamP pElem,
	FHIMatrixRec *pMatrix
)
{
moa_try

	/* variable declarations */
	MoaBoolParam		returnVal;
	
	/* enumerate the path */
	returnVal = pFhCbk->FHIElemPathForAll(pElem, pMatrix, pFhCbk, _FeedbackMoveProc, _FeedbackLineProc, _FeedbackCurveProc, _FeedbackCloseProc);

moa_catch

	/* report the exception */
	MOA_DEBUGSTR( ("Exception thrown at %s.", moa_get_exception_location) );
	returnVal = FALSE;

moa_catch_end

moa_try_end_( MoaBoolParam, returnVal )
}


/*
 *	special (required for 68k)
 */
#if defined(__MWERKS__) && !defined(powerc)
#pragma pop
#endif
