/**********************************************************************
	NOTICE: This code is part of the Altsys Code Library.
			Please don't make any changes to this code unless
			absolutely necessary. If you need to change this
			code, then please contact the Code Librarian before
			doing so. If you do make changes, then make sure
			that they are merged back into the code library.
					Thanks, James Armstrong-Brasure
/**********************************************************************/

/*
	cpgraf.h -- header file for cpgraf.c (cross platform graphics routines)

	Copyright (c) 1994 Altsys. All rights reserved.
*/

#ifndef _H_cpgraf
#define _H_cpgraf

#ifndef _H_moaxtra
	#include "moaxtra.h"
#endif

#ifdef WINDOWS
	#ifndef __WINDOWS__
	#include <windows.h>
	#endif
#endif

/*----------------------------------------------------- constants and types */
/* common colors (CPColor) */
#define CP_COLOR_RED		0x000000FFL
#define CP_COLOR_GREEN		0x0000FF00L
#define CP_COLOR_BLUE		0x00FF0000L
#define CP_COLOR_BLACK		0x00000000L
#define CP_COLOR_WHITE		0x00FFFFFFL
#define CP_COLOR_GRAY		0x00808080L
#define CP_COLOR_LT_GRAY	0x00C0C0C0L

/* font style flags (CPFontStyle) */
#define CP_STYLE_PLAIN		0x00000000L
#define CP_STYLE_BOLD		0x00000001L
#define CP_STYLE_ITALIC		0x00000002L
#define CP_STYLE_UNDERLINE	0x00000004L

/* text alignment modes */
typedef enum {
	CP_ALIGN_LEFT,		/* align text on the left */
	CP_ALIGN_RIGHT,		/* align text on the right */
	CP_ALIGN_CENTER		/* center text */
} _CPAlignMode;
typedef MoaLong CPAlignMode;

/* horizontal positioning modes (used when drawing pictures) */
typedef enum {
	CP_HORZ_STRETCH,	/* stretch picture to fit between left and right */
	CP_HORZ_LEFT,		/* align picture at left */
	CP_HORZ_RIGHT,		/* align picture at right */
	CP_HORZ_CENTER		/* center picture between left and  right */
} _CPHorzPos;
typedef MoaLong CPHorzPos;

/* vertical positioning modes (used when drawing pictures) */
typedef enum {
	CP_VERT_STRETCH,	/* stretch picture to fit between  top and bottom */
	CP_VERT_TOP,		/* align picture at top */
	CP_VERT_BOTTOM,		/* align picure at bottom */
	CP_VERT_CENTER		/* center picture between top and bottom */
} _CPVertPos;
typedef MoaLong CPVertPos;

/* text background modes */
typedef enum {
	CP_TEXT_OPAQUE,		/* bits behind text are wiped out */
	CP_TEXT_TRANSPARENT	/* bits behind text are not wiped out */
} _CPTextMode;
typedef MoaLong CPTextMode;

/* raster modes used for drawing (not text) */
typedef enum {
	CP_RASTER_COPY,		/* pixels behind pen are wiped out */
	CP_RASTER_XOR		/* pixels behind pen are XORed */
} _CPRasterMode;
typedef MoaLong CPRasterMode;

typedef MoaUlong CPFontStyle;
typedef MoaLong CPFixed;	/* a 16.16 FhFixed */
typedef unsigned char CPByte;
typedef XtraResourceCookie CPResFile;

#define CPFIXED_TO_FLOAT(f)	((double)(f) / 65536.0)
#define FLOAT_TO_CPFIXED(f)	((CPFixed)((f) * 65536.0))

#ifdef WINDOWS
	typedef HDC CPPort;
	typedef HWND CPWindow;
	typedef HBITMAP CPPictureHan;
	typedef HICON CPIconHan;
    typedef RECT CPRect;
    typedef POINT CPPoint;
    typedef COLORREF CPColor;
    typedef HRGN CPRegionHan;
#endif

#ifdef MACINTOSH
	typedef GrafPtr CPPort;
	typedef WindowPtr CPWindow;
	typedef PicHandle CPPictureHan;
    typedef Handle CPIconHan;
	typedef Rect CPRect;

	typedef struct {
		short y;
		short x;
	} CPPoint;

	typedef MoaUlong CPColor;
	typedef RgnHandle CPRegionHan;

	#ifndef TRUE
	#define TRUE 1
	#endif
	
	#ifndef FALSE
	#define FALSE 0
	#endif
#endif

/*-------------------------------------------------------------- prototypes */

// CLSID_CPGraf: ABB05BA60017C12E0000080007160DC3
MOA_DEFINE_GUID(CLSID_CPGraf, 0xABB05BA6L, 0x0017, 0xC12E, 0x00, 0x00, 0x08, 0x00, 0x07, 0x16, 0x0D, 0xC3);

#undef INTERFACE
#define INTERFACE IMoaCPGraf

DECLARE_MOA_INTERFACE_(IMoaCPGraf, IMoaUnknown)
{
	STD_IMOAUNKNOWN_METHODS

	STDMOAMETHOD_(void, CPBreakColor) (THIS_ CPColor theColor, CPByte FAR * redP, CPByte FAR * greenP, CPByte FAR * blueP) PURE;
	STDMOAMETHOD_(void, CPClearPattern) (THIS) PURE;
	STDMOAMETHOD_(CPRegionHan, CPGetClipRegion) (THIS) PURE;
	STDMOAMETHOD_(void, CPDrawChar) (THIS_ MoaLong x, MoaLong y, MoaUlong ch, CPTextMode backMode) PURE;
	STDMOAMETHOD_(void, CPDrawIcon) (THIS_ CPIconHan iconH, const CPRect FAR * rectP, MoaLong isColorIcon) PURE;
	STDMOAMETHOD_(MoaLong, CPDrawIconRes) (THIS_ CPResFile resFile, MoaChar FAR * iconName, const CPRect FAR * rectP) PURE;
	STDMOAMETHOD_(void, CPDrawLine) (THIS_ MoaLong x1, MoaLong y1, MoaLong x2, MoaLong y2) PURE;
	STDMOAMETHOD_(void, CPDrawPicture) (THIS_ CPPictureHan pictH, const CPRect FAR * rectP, CPHorzPos horzPos, CPVertPos vertPos, CPRect FAR * finalRectP) PURE;
	STDMOAMETHOD_(MoaLong, CPDrawPictureRes) (THIS_ CPResFile resFile, MoaChar FAR * pictName, const CPRect FAR * rectP, CPHorzPos horzPos, CPVertPos vertPos, CPRect FAR * finalRectP) PURE;
	STDMOAMETHOD_(void, CPDrawText) (THIS_ MoaLong x, MoaLong y, MoaChar FAR * str, CPTextMode backMode) PURE;
	STDMOAMETHOD_(void, CPDrawTextBox) (THIS_ const CPRect FAR * rectP, MoaChar FAR * str, CPTextMode backMode, CPAlignMode align) PURE;
	STDMOAMETHOD_(void, CPEraseOval) (THIS_ const CPRect FAR * rectP) PURE;
	STDMOAMETHOD_(void, CPErasePoly) (THIS_ CPPoint FAR * pointsP, MoaLong numPoints) PURE;
	STDMOAMETHOD_(void, CPErasePort) (THIS) PURE;
	STDMOAMETHOD_(void, CPEraseRect) (THIS_ const CPRect FAR * rectP) PURE;
	STDMOAMETHOD_(void, CPEraseRegion) (THIS_ CPRegionHan regionH) PURE;
	STDMOAMETHOD_(void, CPEraseRoundRect) (THIS_ const CPRect FAR * rectP, MoaLong ovalWidth, MoaLong ovalHeight) PURE;
	STDMOAMETHOD_(void, CPExcludeClipRect) (THIS_ const CPRect FAR * rectP) PURE;
	STDMOAMETHOD_(void, CPFillOval) (THIS_ const CPRect FAR * rectP) PURE;
	STDMOAMETHOD_(void, CPFillPoly) (THIS_ CPPoint FAR * pointsP, MoaLong numPoints) PURE;
	STDMOAMETHOD_(void, CPFillRect) (THIS_ const CPRect FAR * rectP) PURE;
	STDMOAMETHOD_(void, CPFillRegion) (THIS_ CPRegionHan regionH) PURE;
	STDMOAMETHOD_(void, CPFillRoundRect) (THIS_ const CPRect FAR * rectP, MoaLong ovalWidth, MoaLong ovalHeight) PURE;
	STDMOAMETHOD_(void, CPFrameOval) (THIS_ const CPRect FAR * rectP) PURE;
	STDMOAMETHOD_(void, CPFramePoly) (THIS_ CPPoint FAR * pointsP, MoaLong numPoints) PURE;
	STDMOAMETHOD_(void, CPFrameRect) (THIS_ const CPRect FAR * rectP) PURE;
	STDMOAMETHOD_(void, CPFrameRegion) (THIS_ CPRegionHan regionH) PURE;
	STDMOAMETHOD_(void, CPFrameRoundRect) (THIS_ const CPRect FAR * rectP, MoaLong ovalWidth, MoaLong ovalHeight) PURE;
	STDMOAMETHOD_(CPColor, CPGetBackColor) (THIS) PURE;
	STDMOAMETHOD_(MoaLong, CPGetCharWidth) (THIS_ MoaUlong ch) PURE;
	STDMOAMETHOD_(void, CPGetClipRect) (THIS_ CPRect FAR * rectP) PURE;
	STDMOAMETHOD_(MoaLong, CPGetColorDepth) (THIS) PURE;
	STDMOAMETHOD_(void, CPGetFontInfo) (THIS_ MoaChar FAR * fontName, CPFontStyle FAR * styleP, short FAR * ptSizeP) PURE;
	STDMOAMETHOD_(void, CPGetFontMetrics) (THIS_ short FAR * ascentP, short FAR * descentP, short FAR * leadingP, short FAR * lineHeightP) PURE;
	STDMOAMETHOD_(CPPictureHan, CPGetPattern) (THIS) PURE;
	STDMOAMETHOD_(CPPort, CPGetPort) (THIS) PURE;
	STDMOAMETHOD_(void, CPGetPen) (THIS_ short FAR * penWidthP, CPColor FAR * penColorP) PURE;
	STDMOAMETHOD_(CPColor, CPGetPixelColor) (THIS_ MoaLong x, MoaLong y) PURE;
	STDMOAMETHOD_(void, CPGetPortRect) (THIS_ CPRect FAR * rectP) PURE;
	STDMOAMETHOD_(void, CPGetPos) (THIS_ short FAR * xP, short FAR * yP) PURE;
	STDMOAMETHOD_(CPRasterMode, CPGetRasterMode) (THIS) PURE;
	STDMOAMETHOD_(MoaLong, CPGetRectColorDepth) (THIS_ CPRect FAR * rectP, MoaLong getMin) PURE;
	STDMOAMETHOD_(MoaLong, CPGetStringWidth) (THIS_ MoaChar FAR * str) PURE;
	STDMOAMETHOD_(void, CPIntersectClipRect) (THIS_ CPRect FAR * rectP) PURE;
	STDMOAMETHOD_(void, CPInvertOval) (THIS_ const CPRect FAR * rectP) PURE;
	STDMOAMETHOD_(void, CPInvertPoly) (THIS_ CPPoint FAR * pointsP, MoaLong numPoints) PURE;
	STDMOAMETHOD_(void, CPInvertRect) (THIS_ const CPRect FAR * rectP) PURE;
	STDMOAMETHOD_(void, CPInvertRegion) (THIS_ CPRegionHan regionH) PURE;
	STDMOAMETHOD_(void, CPInvertRoundRect) (THIS_ const CPRect FAR * rectP, MoaLong ovalWidth, MoaLong ovalHeight) PURE;
	STDMOAMETHOD_(void, CPLineTo) (THIS_ MoaLong x, MoaLong y) PURE;
	STDMOAMETHOD_(CPColor, CPMakeColor) (THIS_ MoaUlong r, MoaUlong g, MoaUlong b) PURE;
	STDMOAMETHOD_(CPColor, CPMakeGray) (THIS_ CPFixed grayPercent) PURE;
	STDMOAMETHOD_(void, CPMoveTo) (THIS_ MoaLong x, MoaLong y) PURE;
	STDMOAMETHOD_(void, CPReleasePort) (THIS_ CPPort port, CPPort oldPort) PURE;
	STDMOAMETHOD_(void, CPReleaseWin) (THIS_ CPWindow win, CPPort oldPort) PURE;
	STDMOAMETHOD_(void, CPSetBackColor) (THIS_ CPColor backColor) PURE;
	STDMOAMETHOD_(void, CPSetClipRect) (THIS_ CPRect FAR * rectP) PURE;
	STDMOAMETHOD_(void, CPSetClipRegion) (THIS_ CPRegionHan regionH) PURE;
	STDMOAMETHOD_(void, CPSetFont) (THIS_ MoaChar FAR * fontName, CPFontStyle style, MoaLong ptSize) PURE;
	STDMOAMETHOD_(MoaLong, CPSetPattern) (THIS_ CPPictureHan pictH) PURE;
	STDMOAMETHOD_(MoaLong, CPSetPatternRes) (THIS_ CPResFile resFile, MoaChar FAR * pictName) PURE;
	STDMOAMETHOD_(void, CPSetPen) (THIS_ MoaLong penWidth, CPColor penColor) PURE;
	STDMOAMETHOD_(void, CPSetPixelColor) (THIS_ MoaLong x, MoaLong y, CPColor pixelColor) PURE;
	STDMOAMETHOD_(void, CPSetRasterMode) (THIS_ CPRasterMode mode) PURE;
	STDMOAMETHOD_(CPPort, CPUsePort) (THIS_ CPPort port) PURE;
	STDMOAMETHOD_(CPPort, CPUseWin) (THIS_ CPWindow win) PURE;
};
typedef IMoaCPGraf FAR * PIMoaCPGraf;

// IID_IMoaCPGraf: ABB05BA60017C1310000080007160DC3
MOA_DEFINE_GUID(IID_IMoaCPGraf, 0xABB05BA6L, 0x0017, 0xC131, 0x00, 0x00, 0x08, 0x00, 0x07, 0x16, 0x0D, 0xC3);


#undef INTERFACE
#define INTERFACE IMoaCPRes

DECLARE_MOA_INTERFACE_(IMoaCPRes, IMoaUnknown)
{
	STD_IMOAUNKNOWN_METHODS

	STDMOAMETHOD_(void, CPResFreeIcon) (THIS_ CPIconHan FAR * iconHP) PURE;
	STDMOAMETHOD_(void, CPResFreePicture) (THIS_ CPPictureHan FAR * pictureHP) PURE;
	STDMOAMETHOD_(MoaLong, CPResGetIconHeight) (THIS_ CPIconHan iconH) PURE;
	STDMOAMETHOD_(MoaLong, CPResGetIconWidth) (THIS_ CPIconHan iconH) PURE;
	STDMOAMETHOD_(CPIconHan, CPResLoadIcon) (THIS_ CPResFile resFile, const MoaChar FAR * iconName) PURE;
	STDMOAMETHOD_(CPPictureHan, CPResLoadPicture) (THIS_ CPResFile resFile, const MoaChar FAR * pictName, MoaLong id) PURE;
};
typedef IMoaCPRes FAR * PIMoaCPRes;

// IID_IMoaCPRes: ABB311C9000A3D570000080007160DC3
MOA_DEFINE_GUID(IID_IMoaCPRes, 0xABB311C9L, 0x000A, 0x3D57, 0x00, 0x00, 0x08, 0x00, 0x07, 0x16, 0x0D, 0xC3);

#endif /* _H_cpgraf */

/* eof */
