#ifndef _H_moatypes
#define _H_moatypes

#pragma once

/*
	File: moatypes.h (adaptive)

	Copyright (c) 1994, 1995, 1996 Macromedia, Inc.  All Rights Reserved
	
	You may utilize this source file to create and compile object code for use within products 
	you may create.  THIS CODE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, AND 
	MACROMEDIA DISCLAIMS ALL IMPLIED WARRANTIES INCLUDING, BUT NOT LIMITED TO, MERCHANTABILITY, 
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT WILL MACROMEDIA BE 
	LIABLE TO YOU FOR ANY CONSEQUENTIAL, INDIRECT OR INCIDENTAL DAMAGES ARISING OUT OF YOUR 
	USE OR INABILITY TO USE THIS CODE.

	Abstract:
	
	Fundamental types for the MOA Services library.
	
	Revision history:
	
	960609	Release 1.6:

			added kMoaErr_NotImplemented, MoaStringRef

			(srj)

	960609	Release 1.43: No changes. (srj)
	960207	Release 1.43:
	
			Revised the definition of FAR under Win32 to exactly match the Windows
			headers.

			Added MAKE_MOASTATUS to allow for creation of status (non-error) result
			codes.

			added kMoaErr_DeadObject.

			(srj)

	960111	Release 1.42:
			
			Added kMoaErr_DupeClassesWarn.
			WinToMoaPoint macro was wrong.
			
			(srj)
	
	951220	Release 1.41:
			
			changed the definition of MoaError to match HRESULT.
			Under 16-bit windows, this is a void* rather than
			a long. Also changed the MAKE_MOAERR macro to cast
			the value to MoaError (it was a MoaUlong before).

	951128	Release 1.39:
			
			ensure WINDOWS is defined if WIN32 is defined. (srj)
	
	951110	Release 1.38:
			
			reversed the order of x and y in MoaPoint. This allows MoaRect to be treated
			as an array of two MoaPoints.
			
			(srj)
	
	951003	Release 1.3.4:
			
			added the "smart" definitions of WINDOWS and MACINTOSH.
			
			added PMoaDouble, MoaCoord, MoaPoint, MoaRect, and conversion macros.
			
			moved all error codes from moaxtra.h.
			
			(srj)
	
	950922	Release 1.3.2:
			
			added MoaUnichar (unicode character).
			
			(srj)
	
	950901	Release 1.3.1:

			change HUGE to __HUGE.
			
			(srj)	
	
	950811	Release 1.3. (srj)
*/

#if defined(WIN32) && !defined(WINDOWS)
	#define WINDOWS 1
#endif

#if defined(_WINDOWS) && !defined(WINDOWS)
	#define WINDOWS 1
	#if defined(_WIN32) && !defined(WIN32)
		#define WIN32 1
	#endif
#endif

#if !defined(MACINTOSH) && (defined(powerc) || defined(applec))
	#define MACINTOSH 1
#endif

#define MOA_MAX_PATHNAME		260
#define MOA_MAX_FILENAME		256

#ifndef FAR
	#ifdef MACINTOSH
		#define FAR
	#endif
	#ifdef WINDOWS
		#if !defined(_INC_WINDOWS) && !defined(_WINDOWS_)
			#ifdef WIN32
				#define far
				#define FAR		far
			#else
				#define FAR 	_far
			#endif
		#endif
	#endif
#endif 

#ifndef __HUGE
	#ifdef MACINTOSH
		#define __HUGE
	#endif
	#ifdef WINDOWS
		#ifdef WIN32
			#define __HUGE
		#else
			#define __HUGE 	_huge
		#endif
	#endif
#endif

/* ========================================================================== */
/* ======================== Fundamental Constants =========================== */
/* ========================================================================== */

#ifndef TRUE
	#define TRUE 1
#endif
#ifndef FALSE
	#define FALSE 0
#endif

/* ========================================================================== */
/* ============================ Fundamental Types =========================== */
/* ========================================================================== */

typedef char					MoaChar;		/* this is assumed to be signed. */
typedef MoaChar FAR *			PMoaChar;
typedef const MoaChar FAR *		ConstPMoaChar;
typedef unsigned short			MoaUnichar;		/* unicode character */
typedef MoaUnichar FAR *		PMoaUnichar;
typedef const MoaUnichar FAR *	ConstPMoaUnichar;
typedef unsigned char			MoaBool;
typedef MoaBool *				PMoaBool;
typedef unsigned char			MoaByte;
typedef MoaByte *				PMoaByte;
typedef unsigned short			MoaWord;
typedef MoaWord *				PMoaWord;
typedef unsigned long			MoaDword;
typedef MoaDword *				PMoaDword;
typedef short 					MoaShort;
typedef MoaShort *				PMoaShort;
typedef unsigned short 			MoaUshort;
typedef MoaUshort *				PMoaUshort;
typedef long 					MoaLong;
typedef MoaLong *				PMoaLong;
typedef unsigned long 			MoaUlong;
typedef MoaUlong *				PMoaUlong;
typedef long					MoaFixed;		/* 16.16 fixed point number */
typedef MoaFixed *				PMoaFixed;
typedef float 					MoaFloat;		/* 32-bit IEEE */
typedef MoaFloat *				PMoaFloat;
typedef void FAR * 				PMoaVoid;
typedef const void FAR * 		ConstPMoaVoid;
typedef void FAR * FAR *		PPMoaVoid;

/* conditional types */
#if (defined(WINDOWS) || defined(_WINDOWS)) && !(defined(WIN32) || defined(_WIN32))
	typedef void FAR *			MoaError;
#else
	typedef long				MoaError;
#endif
#if	defined(__MWERKS__) && defined(MACINTOSH)
	/* note that "short double" is always 64-bit in Mwerks,
		regardless of "8-byte double" and "Gen 881". */
	typedef short double 		MoaDouble;		/* 64-bit IEEE */
#else
	typedef double 				MoaDouble;		/* 64-bit IEEE */
#endif
typedef MoaDouble FAR *			PMoaDouble;

/* for types < 32 bits: long-sized versions for arg lists */
typedef MoaUlong				MoaUnicharParam;
typedef MoaLong					MoaCharParam;
typedef MoaLong	 				MoaBoolParam;

typedef struct _MoaStringRef FAR* MoaStringRef;

/* ========================================================================== */
/* =============================== "Wide" Types ============================= */
/* ========================================================================== */

#ifdef MACINTOSH
/* Binary match with Apple's "wide" type */
typedef struct MoaWide {
	MoaLong		hi;
	MoaUlong 	lo;
} MoaWide;
#endif

#ifdef WINDOWS
/* Binary match with MS "LARGE_INTEGER" type */
typedef struct MoaWide {
	MoaUlong 	lo;
	MoaLong		hi;
} MoaWide;
#endif

#define LONG_TO_WIDE(L, W)	{ (W).lo = (MoaUlong)(L); (W).hi = (L) >= 0 ? 0 : -1; }
#define WIDE_TO_LONG(W, L)	{ (L) = (MoaLong)(W).lo; }	/* ignores overflow -- use WIDE_FITS */
#define WIDE_FITS(W) \
		(((W).hi == 0 && ((W).lo & 0x80000000) == 0) || \
		((W).hi == -1 && ((W).lo & 0x80000000) != 0))

#define WIDE_GT_ZERO(x)		((x).hi >= 0 && (x).lo != 0)
#define WIDE_EQ_ZERO(x)		((x).hi == 0 && (x).lo == 0)
#define WIDE_LT_ZERO(x)		((x).hi < 0)

/* effects "x += y" where both x and y are MoaWide */
#define WIDE_ADD_WIDE(x, y) \
 	{ MoaUlong tmp = (x).lo; (x).hi += (y).hi; (x).lo += (y).lo; if ((x).lo < tmp) (x).hi++; }

/* effects "x = -x", where x is a MoaWide */
#define WIDE_NEGATE(x) \
	{ (x).hi = ~(x).hi; (x).lo = ~(x).lo; ++(x).lo; if ((x).lo == 0) ++(x).hi; }

/* ========================================================================== */
/* =============================== "Coord" Types ============================ */
/* ========================================================================== */

typedef MoaLong	MoaCoord;

typedef struct MoaPoint {
	MoaCoord y;
	MoaCoord x;
} MoaPoint;

typedef MoaPoint FAR * PMoaPoint;
typedef const MoaPoint FAR * ConstPMoaPoint;

typedef struct MoaRect {
	MoaCoord top;
	MoaCoord left;
	MoaCoord bottom;
	MoaCoord right;
} MoaRect;

typedef MoaRect FAR * PMoaRect;
typedef const MoaRect FAR * ConstPMoaRect;

/* Macros for converting to/from Mac types */
#define	MoaToMacPoint(moa,mac) do { (mac)->h = (short)(moa)->x; (mac)->v = (short)(moa)->y; } while (0)
#define	MacToMoaPoint(mac,moa) do { (moa)->x = (MoaCoord)(mac)->h; (moa)->y = (MoaCoord)(mac)->v; } while (0)

#define	MoaToMacRect(a,b)  	do { \
							(b)->top    = (short)(a)->top;		\
                            (b)->left   = (short)(a)->left;		\
							(b)->bottom = (short)(a)->bottom;	\
							(b)->right  = (short)(a)->right;	\
							} while (0)
							
#define	MacToMoaRect(a,b)  	do { \
							(b)->top    = (MoaCoord)(a)->top;	\
							(b)->left   = (MoaCoord)(a)->left;	\
							(b)->bottom = (MoaCoord)(a)->bottom;\
							(b)->right  = (MoaCoord)(a)->right;	\
							} while (0)

/* Macros for converting to/from Win types */
#define	MoaToWinPoint(moa,win) do { (win)->x = (int)(moa)->x; (win)->y = (int)(moa)->y; } while (0)
#define WinToMoaPoint(win,moa) do { (moa)->x = (MoaCoord)(win)->x; (moa)->y = (MoaCoord)(win)->y; } while (0)

#define	MoaToWinRect(a,b)  	do { \
							(b)->top    = (int)(a)->top;		\
							(b)->left   = (int)(a)->left;		\
							(b)->bottom = (int)(a)->bottom;		\
							(b)->right  = (int)(a)->right;		\
							} while (0)
							
#define	WinToMoaRect(a,b)  	do { \
							(b)->top    = (MoaCoord)(a)->top;	\
							(b)->left   = (MoaCoord)(a)->left;	\
							(b)->bottom = (MoaCoord)(a)->bottom;\
							(b)->right  = (MoaCoord)(a)->right;	\
							} while (0)

/* ========================================================================== */
/* =============================== Utility Macros =========================== */
/* ========================================================================== */

#define MoaMin(a, b)	(((a) < (b)) ? (a) : (b))
#define MoaMax(a, b)	(((a) > (b)) ? (a) : (b))

/* ========================================================================== */
/* ================================== Error codes =========================== */
/* ========================================================================== */

/* these are cunningly crafted to comply with the bit allocation of an HRESULT. */

#define		kMoaErr_NoErr 			((MoaError)0L)

#define 		MAKE_MOAERR(code)		((MoaError)((MoaUlong)(1L << 31) | (MoaUlong)(4L << 16) | (MoaUlong)(code)))

#define 		MAKE_MOASTATUS(code) 	((MoaError)((MoaUlong)(0L << 31) | (MoaUlong)(4L << 16) | (MoaUlong)(code)))

//  I am really bored of the warning messages that say this is re defined....
//  Since this is getting re-defined by winerror.h as the exact same thing, I am removing it 
//  for the PC
#ifdef MACINTOSH
	#ifndef __CFPLUGINCOM__
		#include <CFPluginCOM.h>
	#endif
#endif

#define 		kMoaStatus_OK			((MoaError)0L)
#define 		kMoaStatus_False		((MoaError)1L)

#define		kMoaErr_BadParam				MAKE_MOAERR(1)	/* you passed me a bogus argument */
#define		kMoaErr_OutOfMem				MAKE_MOAERR(2)	/* core "memory full" error */
#define		kMoaErr_XtraInUse				MAKE_MOAERR(3)	/* xtra has objects in use */
#define		kMoaErr_BadInterface			MAKE_MOAERR(4)	/* don't know that interface */
#define		kMoaErr_BadClass				MAKE_MOAERR(5)	/* don't know that class */
#define		kMoaErr_FileNotFound			MAKE_MOAERR(6)	/* couldn't find or open the file */
#define		kMoaErr_XtraNotFound			MAKE_MOAERR(7)	/* couldn't find the Xtra within the file */
#define		kMoaErr_FuncNotFound			MAKE_MOAERR(8)	/* required function missing from Xtra */
#define		kMoaErr_ClassInfoNotFound		MAKE_MOAERR(9)	/* class info missing from Xtra */
#define		kMoaErr_ConnectionNotOpen		MAKE_MOAERR(10)	/* Xtra isn't properly opened (internal error) */

	/* these are standard errors that an Xtra may use 
		if it finds it can't run on a given configuration. It
		should return these from DllGetClassObject/DllGetInterface
		(or the MoaCreatedProc if you are using the standard shell
		macros). */
#define		kMoaErr_NotEnoughCPU			MAKE_MOAERR(11)	/* need better CPU (e.g, >= 68020 */
#define		kMoaErr_NotEnoughFPU			MAKE_MOAERR(12)	/* FPU required for this Xtra */
#define		kMoaErr_NotEnoughHardware		MAKE_MOAERR(13)	/* extra HW needed (e.g., scanner) */
#define		kMoaErr_NotEnoughSSW			MAKE_MOAERR(14)	/* system software is too old */
#define		kMoaErr_NotEnoughExtraSSW		MAKE_MOAERR(15)	/* missing necessary inits (e.g., QuickTime */
#define		kMoaErr_NotEnoughDiskSpace		MAKE_MOAERR(16)	/* need more free space on disk */

#define 	kMoaErr_NoUnusedFreed			MAKE_MOAERR(17)	/* MoaFreeUnused didn't free anything */

#define 	kMoaErr_RequiredCallbackMissing	MAKE_MOAERR(18)	/* a required callback interface
																(e.g., IMoaHandle) was not supplied
																by your callback class */
#define 	kMoaErr_NoSuchKey				MAKE_MOAERR(19)	/* no such key */
#define 	kMoaErr_InternalError			MAKE_MOAERR(20)	/* shouldn't happen */
#define		kMoaErr_UnsupportedCodeType		MAKE_MOAERR(21)	/* Xtra doesn't have kind of code we need */
#define		kMoaErr_DupeClassesWarn			MAKE_MOAERR(22)	/* duplicate class IDs found */
#define		kMoaErr_DeadObject				MAKE_MOAERR(23)	/* object has been disposed */

#define		kMoaErr_Fail					MAKE_MOAERR(24)	//	Unexpected error occurred

#define		kMoaErr_Cancel					MAKE_MOAERR(30)	//	Unexpected error occurred

#define 	kMoaErr_NotImplemented			((MoaError)0x80004001L)		/* same as E_NOTIMPL */

//  SetCustomData tags
#define CUSTOMDATA_URL			"URLTag"
#define CUSTOMDATA_NOTE			"note"
#define CUSTOMDATA_OBJNAME		"SetName"
#define CUSTOMDATA_ACTION		"Action"
#define	CUSTOMDATA_EVENT		"Event"
#define CUSTOMDATA_PARAM1		"Param1"
#define CUSTOMDATA_PARAM2		"Param2"
#define CUSTOMDATA_PARAM3		"Param3"
#define CUSTOMDATA_TARGET		"Target"

//  Action custom data values
enum {
		ACTION_CLEAR = 0,
		ACTION_GOTO,		/* Go To and Stop */
		ACTION_PLAY,
		ACTION_STOP,
		ACTION_PRINT,
		ACTION_FULLSCR,
		ACTION_SSDRAG,
		ACTION_LOAD,
		ACTION_UNLOAD,
		ACTION_TELLTARGET,
		ACTION_GOTOANDPLAY,
		ACTION_GETURL,
		_NUMACTIONS
};

//  Event custom data values
enum {
		EVENT_ON_PRESS = 0,
		EVENT_ON_RELEASE,
		EVENT_ON_OVER,
		EVENT_ON_OUT,
		EVENT_ON_FRAMEACTION
};

#endif /* _H_moatypes */

