/*
	file: muttag.h

	Copyright (c) 1994 Altsys. All rights reserved.
*/

#pragma once

#ifndef _H_muttag
#define _H_muttag


/* Tags for text formatters. */
enum {
	TAGTYPE_UNKNOWN, 
	TAGTYPE_OBJECT, 
	TAGTYPE_BOOL, 
	TAGTYPE_INTEGER,
	TAGTYPE_FIXED
};

#define TAG_RUNTIME				((mushort) 0x4000)		/* add to type in BUILDTAG */

#define BUILDTAG(key, type)		(((key) << 3) + (type))

#define TAG_TF_BASE					600
#define BUILDTAG_TF(a, b)			BUILDTAG(TAG_TF_BASE+(a), (b))


/* 
	Corresponds to "Ignore columns less than" in the inspector.
	Specifies the minimum percent of column height required before
	ADJUSTLEADING will apply.  FIX(1) is 100%, FIXRATIO(1,2) is 
	50%, and so on. 
*/
#define TAG_TF_ADJLEADMIN			BUILDTAG_TF(0, TAGTYPE_FIXED)


/* Copyfitting stuff. */

/* 
	Minimium copyfitting percentage. FIX(1) == 100%, etc. 
*/
#define TAG_TF_ADJPTMAX				BUILDTAG_TF(1, TAGTYPE_FIXED)

/* 
	Maximum copyfitting percentage.  FIX(1) == 100% etc. 
*/
#define TAG_TF_ADJPTMIN				BUILDTAG_TF(2, TAGTYPE_FIXED)

/* 
	The calculated copyfitting percentage.  Read only. 
*/
#define TAG_TF_ADJPTOPT				BUILDTAG_TF(3, TAGTYPE_FIXED)

/* 
	Set this flag to TRUE to cause copyfitting to be recalculated. 
*/
#define TAG_TF_ADJUSTPTSIZE			BUILDTAG_TF(42, TAGTYPE_BOOL + TAG_RUNTIME)

/* 
	Adjust leading will cause the columns of text to be adjusted 
	by adding leading so that the last line in the column is at the
	bottom of the column.  Corresponds to the "Modify Leading" 
	checkbox in the inspector for text. 
*/
#define TAG_TF_ADJUSTLEADING		BUILDTAG_TF(4, TAGTYPE_BOOL)

/* 
	Alignment is used only for TEXT_ON_PATH.  It is the alignment
	of the text on the path. 
*/
#define TAG_TF_ALIGNMENT			BUILDTAG_TF(5, TAGTYPE_INTEGER)

/*
	Use this flag to notify FHIElemPathForAll whether or not to skip
	the path (including the border on text).
*/
#define TAG_TF_SKIPPATH				BUILDTAG_TF(5, TAGTYPE_BOOL+TAG_RUNTIME)

/* 
	Set this to FALSE after all other changes to prevent the text
	from being reflowed.  Not often useful, but available. 
*/
#define TAG_TF_AUTOFLOW				BUILDTAG_TF(6, TAGTYPE_BOOL)


/* 
	Draws and prints the path for text.  Corresponds to 
	"Display Border" checkbox in the inspector. 
*/
#define TAG_TF_SHOWPATH				BUILDTAG_TF(8, TAGTYPE_BOOL)

/* 
	Bottom inset for text in columns. 
*/
#define TAG_TF_BOTTOMINSET			BUILDTAG_TF(9, TAGTYPE_FIXED)

/* 
	Bottom inset for text in a path 
*/
#define TAG_TF_BOTTOMPATHINSET		BUILDTAG_TF(10, TAGTYPE_FIXED)

/* 
	Column height for text in columns.  This is read only for
	text in a path 
*/
#define TAG_TF_COLUMNHEIGHT			BUILDTAG_TF(11, TAGTYPE_FIXED)

/* 
	Specifies the order columns & rows are ordered if there are more
	than one of each.  If true, the text goes down the columns first, 
	if false across the rows first. 
*/
#define TAG_TF_COLUMNMAJOR			BUILDTAG_TF(13, TAGTYPE_BOOL)


#define TAG_TF_COLUMNRULE			BUILDTAG_TF(14, TAGTYPE_INTEGER)

/* 
	Specifies the gutter between columns. 
*/
#define TAG_TF_GUTTER				BUILDTAG_TF(16, TAGTYPE_FIXED)

/* 
	Specifies the upper left coordinate of the text.  Read only for
	text in a path. 
*/
#define TAG_TF_LEFT					BUILDTAG_TF(17, TAGTYPE_FIXED)

/* 
	Left inset for text in columns. 
*/
#define TAG_TF_LEFTINSET			BUILDTAG_TF(18, TAGTYPE_FIXED)

/* 
	Left inset for text in a path. 
*/
#define TAG_TF_LEFTPATHINSET		BUILDTAG_TF(19, TAGTYPE_FIXED)

/* 
	If true, causes the number of lines of text in each of multiple
	columns or rows to be balanced, so that each column has the 
	same number of lines (as much as possible).  Corresponds to the 
	"Balance" checkbox in the inspector. 
*/
#define TAG_TF_LEVELCOLUMNS			BUILDTAG_TF(20, TAGTYPE_BOOL)

/* 
	Number of columns. 
*/
#define TAG_TF_NUMCOLUMNS			BUILDTAG_TF(23, TAGTYPE_INTEGER)

/* 
	Number of rows. 
*/
#define TAG_TF_NUMROWS				BUILDTAG_TF(24, TAGTYPE_INTEGER)

/* 
	Used for text on a path only.  0 is rotate, 1 is skew, 3 is vertical. 
*/
#define TAG_TF_ORIENTATION			BUILDTAG_TF(25, TAGTYPE_INTEGER)

/* 
	Right inset for text in columns. 
*/
#define TAG_TF_RIGHTINSET			BUILDTAG_TF(29, TAGTYPE_FIXED)

/* 
	Right inset for text in a path 
*/
#define TAG_TF_RIGHTPATHINSET		BUILDTAG_TF(30, TAGTYPE_FIXED)

/* 
	Specifies the gutter between rows. 
*/
#define TAG_TF_ROWGUTTER			BUILDTAG_TF(31, TAGTYPE_FIXED)

#define TAG_TF_ROWRULE				BUILDTAG_TF(32, TAGTYPE_INTEGER)

#define TAG_TF_THEPATH				BUILDTAG_TF(34, TAGTYPE_OBJECT)

/* 
	Specifies the upper y coordinate of the text. 
*/
#define TAG_TF_TOP					BUILDTAG_TF(35, TAGTYPE_FIXED)

/*
	Specifies the top inset for text in columns. 
*/
#define TAG_TF_TOPINSET				BUILDTAG_TF(36, TAGTYPE_FIXED)

/* 
	Specifies the top inset for text in a path. 
*/
#define TAG_TF_TOPPATHINSET			BUILDTAG_TF(37, TAGTYPE_FIXED)

/* 
	Read only, not particularly useful. 
*/
#define TAG_TF_TS_END				BUILDTAG_TF(39, TAGTYPE_INTEGER)
#define TAG_TF_TS_START				BUILDTAG_TF(40, TAGTYPE_INTEGER)

/* 
	Specifies the column width for text in columns. 
*/
#define TAG_TF_WRAPWIDTH			BUILDTAG_TF(41, TAGTYPE_FIXED)

/* 
	Specifies the first line leading mode.  See LLM_... below 
*/
#define TAG_TF_FIRST_LLMODE			BUILDTAG_TF(42, TAGTYPE_INTEGER)

/* 
	Specifies the value for first line leading, depending on mode.  
*/
#define TAG_TF_FIRST_LEADING		BUILDTAG_TF(43, TAGTYPE_FIXED)

/* 
	Same as ALIGNMENT for text on a path, but for the second line 
*/
#define TAG_TF_ALIGNMENT2			BUILDTAG_TF(44, TAGTYPE_INTEGER)

/* 
	Alignment values for TAG_TF_ALIGNMENT 
*/
#define ALIGN_LEFT		0
#define ALIGN_RIGHT		1
#define ALIGN_CENTER	2
#define ALIGN_JUSTIFY	3

/* 
	The LLM's specify the meaning of the leading value for lines. 
*/
#define LLM_DEFAULT		-1	/* Used to not override the paragraph value */
#define LLM_EXTRA		0	/* Line leading extra leading. (in points)*/
#define LLM_FIXED		1	/* Fixed leading (in points) */
#define LLM_PERCENT		2	/* Percentage leading (120% is auto, 100% is solid, and so on) */
#define LLM_PERCENT_EM	3	/* Special case for first line leading.  Means % of 80% of em.  better 
								value than the current, which is % of ascent, which can be hokey. */

#define	TAG_TF_JP_ORIENT			BUILDTAG_TF(48, TAGTYPE_INTEGER)	
#endif
