/*
	file: muttypes.h

	Copyright (c) 1996 Macromedia, Inc. All rights reserved.
*/

#ifndef _H_muttypes
#define _H_muttypes

#ifndef _H_mutbase
	#include "mutbase.h"
#endif

#ifndef _H_uibase
	#include "uibase.h"
#endif

#ifndef _H_moastdif
	#include "moastdif.h"
#endif

#ifdef WINDOWS
	#pragma pack(push, 2)
#endif

/*
	Please note carefully: the argument to FIX
	must be in the range -32768...32767.
*/
#ifndef FIX
	#define FIX(i)				((MoaFixed) ((MoaLong) (i) << 16))
#endif

/*
	Please note carefully: both arguments to FIXRATIO
	must be in the range -32768...32767.
*/
#ifndef FIXRATIO
	#define FIXRATIO(n, d)		((((long) (n)) << 16) / (d))
#endif

/* ------------------------------------ Argument types for compatibility. */

struct _CursorparamP;
struct _StreamID;
typedef struct _CursorparamP *CursorparamP;
typedef struct _StreamID *StreamID;

typedef void (*FHICloseProc)(PMoaVoid data);
typedef void (*FHIMoveProc)(ConstPMPoint, PMoaVoid data);
typedef void (*FHILineProc)(ConstPMPoint, PMoaVoid data);
typedef void (*FHICurveProc)(ConstPMPoint, ConstPMPoint, ConstPMPoint, PMoaVoid data); 
typedef void (*FHIFSArrowProc)(MoaBoolParam, MoaBoolParam, MoaFixed, MoaLong, MoaLong, PMoaVoid data); 
typedef void (*FHIFillStrokeProc)(MoaBoolParam, MoaBoolParam, PMoaVoid data); 
typedef void (*FHICapJoinProc)(MoaBoolParam, MoaBoolParam, MoaFixed, MoaLong, MoaLong, PMoaVoid data); 
typedef MoaBoolParam (*FHIDataProc)(MoaBoolParam reading, MoaLong* positionP, MoaLong* countP, PMoaVoid buffer, PMoaVoid userData);

/* ------------------------- more types */

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	MoaLong theData[7];
} FHIMatrixRec;

#ifdef powerc
#pragma options align=reset
#endif

/* 
	Some opaque data types.
*/
struct COLORPARAMP;
typedef struct COLORPARAMP *ColorparamP;

struct FILLPARAMP;
typedef struct FILLPARAMP *FillparamP;

struct STROKEPARAMP;
typedef struct STROKEPARAMP *StrokeparamP;

struct ARROWPARAMP;
typedef struct ARROWPARAMP *ArrowparamP;

struct DOCPARAMP;
typedef struct DOCPARAMP *DocparamP;

#ifdef FEATURE_NO_BAD_EXCHANGES
struct ObjectHandle
{
	oid				theObject;
	bool			inDatabase;
	DocparamP		theRefDoc;
#ifdef DEBUG
	ObjectHandle*	pHandle;
#endif
};
typedef ObjectHandle* ElemparamP;
#else
struct ELEMPARAMP;
typedef struct ELEMPARAMP *ElemparamP;
#endif		// FEATURE_NO_BAD_EXCHANGES

struct OUTLINEPARAMP;
typedef struct OUTLINEPARAMP *OutlineparamP;

struct FONTPARAMP;
typedef struct FONTPARAMP *FontparamP;

struct TEXTEFFECTP;
typedef struct TEXTEFFECTP *TexteffectP;

struct LISTPARAMP;
typedef struct LISTPARAMP *ListparamP;

struct PROCEDUREPARAMP;
typedef struct PROCEDUREPARAMP *ProcedureParamP;

/* -------------------------------------------------------- charAttrsRec */
/* Text character attribute record. */
/* 
	The charAttrsRec is used for retrieving character attributes, and for setting
	character attributes.  When used for retrieving with FHITextGetCharAttrs,
	the setFill, setStroke, set... bools are all set to FALSE, and not used
	otherwise.  When used for setting attributes with FHITextSetCharAttrs 
	the set... fields determine which fields get set.  If the value for 
	setFill is FALSE, the fill will not be changed for the text.  If the
	value for setFill is TRUE, the fill will be applied to the text.  This allows
	you to set the baseline shift, for example, but leave the point size and leading
	unchanged.  By setting more than one set... field to TRUE, you can set multiple
	attributes with one call. 
*/

enum {
	EXTRA_LEADING = 0,
	FIXED_LEADING,
	PERCENT_LEADING
};
typedef MoaShort LeadingT;

enum {
	FACE_PLAIN = 0,
	FACE_BOLD,
	FACE_ITALIC,
	FACE_BOLDITALIC
};  
typedef MoaShort FaceT;

typedef FhFxPoint MPoint;
typedef MPoint* PMPoint;

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	MoaChar			family[256];		/* Font family name (times, geneva, arial etc.)  */
	MoaChar			psName[256];		/* Postscript name */
	FillparamP		fill; 				/* The fill for the characters.  Use a basicFill. */
	StrokeparamP	stroke; 			/* The stroke for the characters.  Use a basicStroke. */
	MoaFixed		horizScaling; 		/* Horizontal scaling.  FIX(1) == 100% */
	MoaFixed		leading;  			/* Leading value.  See leadingMode above for an interpretation. */
	MoaFixed		baselineShift; 		/* Baseline shift in points. */
	MoaFixed		rangeKerning; 		/* Range kerning value in percent of the current point size.
								 			FIX(1) would be 100%, and enormous.  0 is the default, and
								 			FIXRATIO(1,10) (10%) is the sort of adjustment that would be sensible. */
	MoaFixed		pointSize;			/* Font point size */
	FaceT			face;				/* Font face style */
	LeadingT	 	leadingMode; 		/* This determines how the leading value is interpreted. 
											If leadingMode == extraLeading, the leading value is the number of 
											points to add to the point size to determine the leading.  FIX(4) 
											would add four points to the current point size.
											If leadingMode == fixedLeading, the leading value is the number of
											points to lead the next line.  FIX(24) would be 24 point leading.
											If leading == percentLeading, the leading value is the percent of the
											current point to lead.  FIX(1) is 100%. */
	MoaBool			keepOnSameLine; 	/* If true, causes characters to become non-breaking.
								 			Using this on a space causes it to behave like a
											non-breaking space.  This should be used with
											caution, as it can cause very ugly word wrap. */
	MoaBool			dontHyphenate; 		/* Disables automatic hyphenation */

	MoaBool			setFill; 			/* If true, apply the fill. */
	MoaBool			setStroke; 			/* If true, apply the stroke. */
	MoaBool			setKeepOnSameLine;	/* If true, apply keepOnSameLine. */
	MoaBool			setDontHyphenate; 	/* If true, apply dontHyphenate. */
	MoaBool			setHorizScaling; 	/* If true, apply horizScaling. */
	MoaBool			setLeadingMode; 	/* If true, apply the leadingMode value. */
	MoaBool			setLeading; 		/* If true, apply the leading value. */
	MoaBool			setBaselineShift; 	/* If true, apply the baselineShift. */
	MoaBool			setRangeKerning; 	/* If true, apply the range kerning. */
	MoaBool			setFace;
	MoaBool			setPointSize;
	MoaBool			setFamily;
	MoaBool			setPsName;			/* if true, set the Postscript name */

	/* Read only fields */
	MoaBool			isTrueType;			/* is a TrueType font. */
	MoaBool			isMacVec;			/* if you don't know what this means, you don't need to know. */
	MoaBool			usesStandardEncoding;

} CharAttrsRec;

#ifdef powerc
#pragma options align=reset
#endif

#ifdef powerc
#pragma options align=mac68k
#endif

/* -------------------------------------------------------- ParaAttrsRec */
/* 
	Spacing range values for word and letter spacing.
	The values are factors, FIX(1) is 100%, 
	FIXRATIO(1,2) is 50%, etc. 
*/
typedef struct {
	MoaFixed min;
	MoaFixed optimum;
	MoaFixed max;
} RangeT;

#ifdef powerc
#pragma options align=reset
#endif

#ifdef powerc
#pragma options align=mac68k
#endif

/* Paragraph margins and indents information.  The values
	are in points. */
typedef struct {
	MoaFixed firstLineIndent;
	MoaFixed left;
	MoaFixed right;
} MarginT;

#ifdef powerc
#pragma options align=reset
#endif

#ifdef powerc
#pragma options align=mac68k
#endif

/* Extra spacing between paragraphs */
typedef struct {
	MoaFixed before;
	MoaFixed after;
} SpacingT;

#ifdef powerc
#pragma options align=reset
#endif

enum {
	alignLeftX = 0, 
	alignRightX, 
	alignCenterX, 
	alignJustifyX
};
typedef MoaShort TextAlignT;

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	MarginT 		margins;
	SpacingT		spacing; 				/* Spacing before & after the paragraph */
	RangeT			extraWordSpacing; 		/* Spacing ranges for word spacing. */
	RangeT			extraLetterSpacing; 	/* Spacing ranges for letter spacing. */
	MoaFixed		flushZone; 				/* Area where the last line in a paragraph will
												be force justified.  Specified as a percent of
												the line width.  FIXRATIO(75,100), 75% indicates
												that last lines whose length is 75% or more of 
												the column width will be force justified if 
												justification is turned on. */
	MoaFixed		raggedWidth; 			/* Sets a minimum line width for flush left,
												right, or centered text.  If it is 90%, then
												lines less than 90% of the column width will
												be justified out to 90% of the column width. */
	TextAlignT		align;
	MoaShort		keepLinesTogether; 		/* Combination of widow & orphan control.
												Specifies the minimum number of lines
												in a paragraph that will be allowed to
												be at the beginning or end of a column
												in multi-colunn text. */
	MoaBool			autoHyphenate; 			/* Enables auto-hyphenation for the paragraph. */
	MoaBool			dontHyphenateCaps; 		/* Disables auto-hyphenation for words that start with a capital letter. */

	MoaBool			setAlign;
	MoaBool			setMargins;
	MoaBool			setSpacing;
	MoaBool			setExtraWordSpacing;
	MoaBool 		setExtraLetterSpacing;
	MoaBool			setFlushZone;
	MoaBool			setKeepLinesTogether;
	MoaBool			setRaggedWidth;
	MoaBool			setAutoHyphenate;
	MoaBool			setDontHyphenateCaps;
} ParaAttrsRec;

#ifdef powerc
#pragma options align=reset
#endif

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	FHIMatrixRec	fhMatrix;
	FhFxPoint			location;
	MoaFixed		length;
	MoaLong			firstChar;
	MoaLong			numChars;
	MoaFixed		descent;
	MoaFixed		maxPointSize;
	MoaFixed		leading;
	MoaShort		flags;
	MoaShort		leadingMode;
	MoaBool			hasMatrix;
} FHILineInfoRec;

#ifdef powerc
#pragma options align=reset
#endif

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	MoaLong version;	/* structure version */

	// Optimization
	MoaLong		lOptimization;
	MoaLong		lPathCompression;
	MoaLong		lImageCompression;
	MoaLong		lTextOutput;

	// Export Options
	MoaLong		lAnimation;
	MoaBool		bAllPages;
	MoaLong		lStartPage;
	MoaLong		lEndPage;

	// Movie Properties
	MoaBool		bAutoPlay;
	MoaBool		bFullScreen;
	MoaBool		bImportProtect;
	MoaBool		bTraceDashes;
	MoaBool		bAllowPrinting;
	MoaFixed	fxFrameRate;

	// New Properties
	MoaBool		bSizeToContents;
	MoaBool		bCompressMovie;

} FlashOutputInfo;

#ifdef powerc
#pragma options align=reset
#endif

#ifdef MACINTOSH

typedef int FHMenuBase;

//  DIRECTLY PULLED FROM FHMENUDEFS.H
// Menu IDS -
#define FH_APPLE_MENU					9999
#define FH_FILE_MENU					10000
#define FH_EDIT_MENU					10001
#define FH_VIEW_MENU					10002
#define FH_MODIFY_MENU					10003
#define FH_TEXT_MENU					10004
#define FH_WINDOW_MENU					10005
#define FH_XTRAS_MENU					10006
#ifdef FEATURE_FLASH_PLAYER
#define FH_CONTROL_MENU					10007
#endif // FEATURE_FLASH_PLAYER

#endif // Macintosh
#ifdef WINDOWS
typedef char * FHMenuBase;
#endif // Windows

typedef struct FHMenuPath {
	FHMenuBase baseMenu;
	char *subMenuName;
	char *commandName;
	int locationInMenu;
} FHMenuPath;
typedef FHMenuPath *PFhMenuPath;

/* --------------------------------------------------------------------- */
/* ---------------------- New font related stuff ----------------------- */
/* --------------------------------------------------------------------- */

/* ---------------------------------------------------------- GetFontRec */

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	MoaChar		psName[256];
	MoaChar		family[256];
	FaceT		face;
	MoaBool		usePsName;
} GetFontRec;

#ifdef powerc
#pragma options align=reset
#endif

#define FMR_MAX_NUM_CHARS		1024

/* ------------------------------------------------------ FontMetricsRec */

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
		PFhFxPoint		widths;
		MoaLong*	firstChar;      /* pointer to ranges of first characters */
		MoaLong*	lastChar;       /* pointer to ranges of last characters */
		/* Number of first/last character pairs.
			always 1 in the first implementation. */
		MoaLong		rangesOfChars;

		// internal use
		FhFxPoint		dataWid[FMR_MAX_NUM_CHARS];
		MoaLong		dataRange[2];
		// end internal use

		MoaFixed	ascent;
		MoaFixed	descent;
		MoaFixed	capHeight;
		MoaFixed	fontBBox[4];
		MoaFixed	italicAngle;
		MoaFixed	vStem;  /* vertical stem widths */

		/* optional for PDF */
		FhFxPoint		avgWidth;	/* optional */
		MoaFixed	leading;	/* optional */
		FhFxPoint		maxWidth;	/* optional */
		FhFxPoint		missingWidth;	/* optional */
		MoaFixed	hStem;		/* horizontal stem widths - optional */
		MoaFixed	xHeight;	/* optional */

		MoaChar		psName[256];
		MoaChar		family[256];
		MoaChar		ttName[256];
		FaceT		face;

		MoaBool		isTrueType;
		MoaBool		isMacVec;

		MoaBool		fixedWidthFont;
		MoaBool		serifFont;
		MoaBool		symbolicFont;
		MoaBool		scriptFont;
		MoaBool		usesStandardEncoding;
		MoaBool		allCapFont;
		MoaBool		smallCapFont;
		MoaBool		forceBold;

		/* Which fields were returned.
			Unmentioned fields should always have good values. */
		MoaBool		capHeightFound;
		MoaBool		fontBBoxFound;
		MoaBool		vStemFound;
		MoaBool		hStemFound;
		MoaBool		avgWidthFound;
		MoaBool		leadingFound;
		MoaBool		maxWidthFound;
		MoaBool		missingWidthFound;
		MoaBool		xHeightFound;
		MoaBool		italicAngleFound;
} FontMetricsRec;

#ifdef powerc
#pragma options align=reset
#endif

/* flags possible ignored in the first version */
#define FHMF_DEFAULT		0
#define FHMF_HORIZONTAL		1
#define FHMF_VERTICAL		2
#define FHMF_GET_OTHER		4

/* --------------------------------------------------------------------- */
/* -------------------- End New font related stuff --------------------- */
/* --------------------------------------------------------------------- */

/* --------------------------------------------------------------------- */
/* --------------------- New text traversal stuff ---------------------- */
/* --------------------------------------------------------------------- */

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	FHIMatrixRec	fhMatrix;
	FhRect			bounds;
	FhFxPoint			location;
	MoaFixed		length;
	MoaLong			firstChar;
	MoaLong			numChars;
	MoaFixed		descent;
	MoaFixed		maxPointSize;
	MoaFixed		leading;
	MoaFixed		measuredLen;
	MoaShort		flags;
	MoaShort		leadingMode;
	MoaShort		alignment;
	MoaUnichar		leaderChar;
	MoaShort		hasMatrix;
} FH60LineInfoRec;

#ifdef powerc
#pragma options align=reset
#endif

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	MoaFixed		version;		/* probably won't be used. FIX_1 is the default */

	ElemparamP		textElem;		/* The text element being traversed */

	MoaLong			position;		/* Initialize this to 0 */
	MoaLong			leaderCharsDone;/* number of leader characters finished */
	MoaLong			lineNumber;
	MoaLong			numLines;
	MoaLong			bufferLen;
	MoaShort*		buffer;			/* if handles unicode (NULL in version 1) */
	MoaChar*		sbuffer;		/* if unicode not handled */
	FhFxPoint			location;		/* coordinate local to text block */
	FhFxPoint			new_location;	/* after the text as been written out */
	FhFxPoint			textWidth;		/* The width of the current buffer */
	FhFxPoint			extraCharSpacing;		/* Currently being used */
	FhFxPoint			extraWordSpacing;		/* Currently being used */
	FhFxPoint			extraDblByteSpacing;	/* Currently being used */
	PFhFxPoint			kernPairs;		/* list of kern pairs */
	MoaLong*		kernPositions;	/* list of positions in the buffer (end with -1) */

	/* Optionally the xtra could grab these fields seperately */
	/* All these fields are in use by the current buffer of text */
	StrokeparamP	strokeP;
	FillparamP		fillP;
	FH60LineInfoRec	lineInfo;
	FH60LineInfoRec	nextLineInfo;
	CharAttrsRec	charAttrs;
	ParaAttrsRec	paraAttrs;
	ElemparamP		graphicP;		/* Setup for inline graphics */
	FontparamP		fontP;
	TexteffectP		textEffectP;
	MoaFixed		pointSize;
	MoaFixed		horizScale;
	MoaFixed		vertScale;
	MoaFixed		textRise;

	FHIMatrixRec	mtx;			/* setup for graphics and text along a path */

	/* These flags tell the xtra what has changed */
	MoaLong			numOfkernPairs;
	MoaBool			isVerticalText;	/* Always FALSE in version 1 */
	MoaBool			locationChanged;
	MoaBool			charSpacingChanged;
	MoaBool			wordSpacingChanged;
	MoaBool			strokeChanged;
	MoaBool			fillChanged;
	MoaBool			lineInfoChanged;
	MoaBool			charAttrsChanged;
	MoaBool			paraAttrsChanged;
	MoaBool			textIsGraphic;
	MoaBool			fontChanged;
	MoaBool			pointSizeChanged;
	MoaBool			matrixChanged;

	MoaBool			horizScaleChanged;
	MoaBool			vertScaleChanged;
	MoaBool			textRiseChanged;
	MoaBool			textEffectChanged;

	/* These flags allow the xtra to tell us what it wants.
		It is not guaranteed that these changes will not be sent to the
		xtra if these are set to FALSE. Be prepare to ignore these changes
		and output the text anyway.
	*/
	MoaBool			takeUnicode;			/* Probably not in version 1 */
	MoaBool			takeVerticalText;		/* Always FALSE in version 1 */
	MoaBool			takeLocationChanged;	/* Always TRUE */

	/* If any of these fields are FALSE then they are sent as location changes */
	MoaBool			takeKernPairsExists;	/* */
	MoaBool			takeCharSpacingChanged;	/* always TRUE in version 1 */
	MoaBool			takeWordSpacingChanged;	/* always TRUE in version 1 */

	MoaBool			takeStrokeChanged;
	MoaBool			takeFillChanged;

	MoaBool			takeLineInfoChanged;
	MoaBool			takeCharAttrsChanged;
	MoaBool			takeParaAttrsChanged;
	MoaBool			takeTextIsGraphic;
	MoaBool			takeFontChanged;
	MoaBool			takePointSizeChanged;
	MoaBool			takeMatrixChanged;

	MoaBool			takeHorizScale;
	MoaBool			takeVertScale;
	MoaBool			takeTextRise;
	MoaBool			takeTextEffects;

	MoaBool			didLeaderText;

	// ignore the man behind the curtain and the data below.
	FhFxPoint			kernPair;
	MoaLong			kernPairPosition;
} TextElemTravRec;

#ifdef powerc
#pragma options align=reset
#endif

/* --------------------------------------------------------------------- */
/* ------------------- End New text traversal stuff -------------------- */
/* --------------------------------------------------------------------- */

/* Standard doc colors  */
enum {
	COLOR_BLACK_X = 0, 
	COLOR_WHITE_X, 
	COLOR_REGISTRATION_X
};

/* --------------------------------------- Fill and stroke info records. */

enum {
	FILL_NONE = 0, 
	FILL_BASIC, 
	FILL_GRADUATED, 
	FILL_PATTERNED, 
	FILL_POSTSCRIPT,
	FILL_RADIAL, 
	FILL_UNKNOWN,
	FILL_TILED,
	FILL_EXT_GRADUATED,  /* extended grad for overprint/multicolor */
	FILL_EXT_RADIAL,   /* extended rad for overprint/multicolor */
	FILL_LENS,
	FILL_CONTOUR
};

enum {
	STROKE_NONE = 0, 
	STROKE_BASIC, 
	STROKE_PATTERNED, 
	STROKE_POSTSCRIPT, 
	STROKE_UNKNOWN,
	STROKE_BRUSH
};

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	ColorparamP theFillColor;
	MoaBool		overprint; 		/* Overprint flag for printing separations. */
} BasicFillRec;

#ifdef powerc
#pragma options align=reset
#endif

enum {
	BUTT_CAP = 0,
	ROUND_CAP,
	SQUARE_CAP
};
typedef MoaShort CapT;

enum {
	MITER_JOIN = 0, 
	ROUND_JOIN, 
	BEVEL_JOIN
};
typedef MoaShort JoinT;

#define MAX_DASH	8

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	ColorparamP theStrokeColor;
	MoaFixed	strokeWidth; 			/* Width of the stroke. */
	MoaFixed 	miterLimit;  			/* A good value is FIXRATIO(386,100)  3.86 about 30 degrees. */
	MoaFixed	dashArray[MAX_DASH];	/* dash components. */
	MoaLong		dashComponents;			/* Dash pattern. 0 is solid. */
	ArrowparamP	theArrow1;				/* First arrowhead.  NULL is ok */
	ArrowparamP	theArrow2;				/* Second arrowhead.  NULL is ok. */
	CapT 		cap;
	JoinT 		join;
	MoaBool		overprint; 				/* Overprint flag for printing separations. */
} BasicStrokeRec;

#ifdef powerc
#pragma options align=reset
#endif

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	ColorparamP	fromColor;	/* Starting color. */
	ColorparamP	toColor;	/* Ending color */
	MoaFixed	angle;		/* Fixed angle in degrees.  FIX(90) is 90 degrees. */
	MoaBool		logarithmic;/* TRUE logarithmic, FALSE linear. */
} GradFillRec;

#ifdef powerc
#pragma options align=reset
#endif

#define MCLIST_MAX_NUM_COLORS 32

#ifdef powerc
#pragma options align=mac68k
#endif

/* the first 4 fields must be identical to the GradFillrect */
typedef struct {
	ColorparamP	fromColor;		/* Starting color. */
	ColorparamP	toColor;		/* Ending color */
	MoaFixed	angle;			/* Fixed angle in degrees.  FIX(90) is 90 degrees. */
	MoaBool		logarithmic;	/* TRUE logarithmic, FALSE linear. */
	MoaBool		reserved1;
	MoaBool		reserved2;
	MoaBool		reserved3;
	ColorparamP	midColors[MCLIST_MAX_NUM_COLORS - 2]; /* an array of ColorparamP's which are the interior colors*/
	MoaFixed midPositions[MCLIST_MAX_NUM_COLORS - 2];/* an array of MoaFixed's which are the interior colors' positions*/
	MoaFixed midCenters[MCLIST_MAX_NUM_COLORS];/* an array of MoaFixed's which are the interior colors' grad centers*/
									/* unused for FH6, for future compatibility */
									/* note, midCenters has one more field than the other arrays */
	MoaShort	numColors;		/* The total number of colors - must always be >= 2 */
	MoaBool		overprint;		/* TRUE overprint, FALSE not overprint. */
	MoaBool		mbNewLinear;
	FhFxPoint	mCenter;
	FhFxPoint	mHandle;
	MoaShort	mFillParm;
	MoaShort	mCount;
} ExtGradFillRec;

#ifdef powerc
#pragma options align=reset
#endif

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	ColorparamP theFillColor;
	MoaChar		pattern[8];		/* the pattern.  It is an 8 x 8 monochrome bitmap,
									stored in 8 bytes.  First byte is top, last byte is 
									bottom.  Most significant bit is left, least significant
									bit is right. */
} PatternFillRec;

#ifdef powerc
#pragma options align=reset
#endif

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	ColorparamP theStrokeColor;
	MoaFixed		strokeWidth;
	MoaChar		pattern[8];		/* the pattern.  It is an 8 x 8 monochrome bitmap,
									stored in 8 bytes.  First byte is top, last byte is 
									bottom.  Most significant bit is left, least significant
									bit is right. */
} PatternStrokeRec;

#ifdef powerc
#pragma options align=reset
#endif

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	ColorparamP	outsideColor;	/* edge color. */
	ColorparamP	centerColor;	/* center color */
	FhFxPoint		center;		/* Center of the fill, x & y values range
							from FIX_0 to FIX_1.  FIXRATIO(1,2) is
							centered, FIX_0 is left (or bottom), 
							FIX_1 is right (or top). */
} RadialFillRec;

#ifdef powerc
#pragma options align=reset
#endif

#ifdef powerc
#pragma options align=mac68k
#endif

/* the first 3 fields must be identical to the GradFillrect */
typedef struct {
	ColorparamP	outsideColor;	/* edge color. */
	ColorparamP	centerColor;	/* center color */
	FhFxPoint		center;		/* Center of the fill, x & y values range
							from FIX_0 to FIX_1.  FIXRATIO(1,2) is
							centered, FIX_0 is left (or bottom), 
							FIX_1 is right (or top). */
	ColorparamP	midColors[MCLIST_MAX_NUM_COLORS - 2]; /* an array of ColorparamP's which are the interior colors*/
	MoaFixed midPositions[MCLIST_MAX_NUM_COLORS - 2];/* an array of MoaFixed's which are the interior colors' positions*/
	MoaFixed midCenters[MCLIST_MAX_NUM_COLORS];/* an array of MoaFixed's which are the interior colors' grad centers*/
									/* unused for FH6, for future compatibility */
									/* note, midCenters has one more field than the other arrays */
	MoaBool		overprint;/* TRUE overprint, FALSE not overprint. */
	MoaShort	numColors;	/* The total number of colors - must always be >= 2 */
} ExtRadialFillRec;

typedef struct {
	ExtRadialFillRec mrExtRadialFillRec;
	MoaShort	msTaper;
	MoaShort	msFillType;
	FhFxPoint	mHandle1;
	FhFxPoint	mHandle2;
	MoaShort	mFillParm;
	MoaShort	mCount;
} ContourFillRec;

enum {CONTOUR_TYPE, NEWCONTOUR_TYPE, RADIAL_TYPE, 
										RECT_TYPE, CONE_TYPE};
#ifdef powerc
#pragma options align=reset
#endif

#ifdef powerc
#pragma options align=mac68k
#endif

/* TiledFillRec, new for FH7 */
typedef struct {
	ElemparamP		tileElems;		/* the group containing the tile elements */
	MoaFixed		hScale;			/* the horizontal scale factor */
	MoaFixed		vScale;			/* the vertical scale factor */
	MoaFixed		rotationAngle;	/* rotation angle in degrees; FIX(90) is 90 degrees */
	FhFxPoint			tileOffset;		/* the tile pattern offset */
	FHIMatrixRec	xformMatrix;	/* a matrix used to represent other transformations
								   which aren't scale, translation, or rotation. though
								   those 3 transformations can be represented in this
								   matrix, ordinarily they should be stored in the other
								   fields and this matrix should be used to represent
								   skews */
} TiledFillRec;

#ifdef powerc
#pragma options align=reset
#endif

#ifdef powerc
#pragma options align=mac68k
#endif

/* TiledFillOutputRec, new for FH7 */
typedef struct {
	ElemparamP		tileElems;		/* the group containing the tile elements */
	MoaFixed		xTileOffset;	/* Offset each tile by this amount (x value) */
	MoaFixed		yTileOffset;	/* (y value) */
	FHIMatrixRec	xformMatrix;	/* All the transformations packed into one place */
	MoaLong			columns;		/* The number of horizontal tiles required to fill the bbox */
	MoaLong			rows;			/* vertical tiles */
} TiledFillOutputRec;

#ifdef powerc
#pragma options align=reset
#endif

/* ------------------------------------ Page info types  */

#define PAGEINFOREC_VERSION		0

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	MoaLong 	version;		/* current version = 0 */
	FhFxPoint		lo;
	FhFxPoint		hi;
	MoaFixed	bleed;
} PageInfoRec;

#ifdef powerc
#pragma options align=reset
#endif

/* ------------------------------------ Paste align type  */
enum { 
	PASTE_ALIGN_UPPER_LEFT = 0, 
	PASTE_ALIGN_CENTER,
	PASTE_SCALE 
};
typedef MoaShort AlignT;

/* ------------------------------------ Along a path types */
enum {
	PATH_ROTATE = 0,
	PATH_VERTICAL, 
	PATH_SKEW
};

/* ------------------------------------ Warping types */
/* 
	Warping takes a rectangular area (specified by 
	lo and hi in the warp record) and maps the space to a 
	warped area specified by a closed curved path.  The initial
	curved path (which will produce output the same as the 
	input) is the original rectangle with one anchor point
	added in the middle of each side, and control points 
	distributed 1/3 of the distance along each line. The 
	anchor points and control points can be changed arbitrarily
	to warp the data.  */

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	FhFxPoint		lo;				/* Lower left bounds of the input data. */
	FhFxPoint		hi;  			/* Upper right bounds of the input data. */
	FhFxPoint		warpFrame[24]; 	/* Closed path composed of 8 connected bezier segments.
								See the WF_... enum that describes each point. */
} FHIWarpRec;

#ifdef powerc
#pragma options align=reset
#endif

#define WF_LOWER_LEFT 					0	/* Lower left of the warp frame. */
#define WF_LOWER_LEFT_MID_LEFT_BCP1 	1	/* First control point between lower left & middle left. */
#define WF_LOWER_LEFT_MID_LEFT_BCP2 	2	/* Second control point between lower left & middle left. */
#define WF_MID_LEFT 					3	/* Anchor point between lower left & upper left. */
#define WF_MID_LEFT_UPPER_LEFT_BCP1 	4	/* First control point between middle left & upper left. */
#define WF_MID_LEFT_UPPER_LEFT_BCP2 	5	/* Second control point between middle left & upper left. */
#define WF_UPPER_LEFT 					6	/* Upper left of warp frame. */

#define WF_UPPER_LEFT_UPPER_MID_BCP1 	7	/* First control point between upper left & upper middle. */
#define WF_UPPER_LEFT_UPPER_MID_BCP2 	8	/* Second control point between upper left & upper middle. */
#define WF_UPPER_MID 					9	/* Anchor point between upper left & upper right. */
#define WF_UPPER_MID_UPPER_RIGHT_BCP1	10	/* First control point between upper middle & upper right. */
#define WF_UPPER_MID_UPPER_RIGHT_BCP2 	11	/* Second control point between upper middle & upper right. */
#define WF_UPPER_RIGHT 					12	/* Upper right of warp frame. */

#define WF_UPPER_RIGHT_MID_RIGHT_BCP1 	13	/* First control point between upper right & middle right. */
#define WF_UPPER_RIGHT_MID_RIGHT_BCP2 	14	/* Second control point between upper right & middle right. */
#define WF_MID_RIGHT 					15	/* Anchor point between upper left & lower right. */
#define WF_MID_RIGHT_LOWER_RIGHT_BCP1 	16	/* First control point between middle right & lower right. */
#define WF_MID_RIGHT_LOWER_RIGHT_BCP2 	17	/* Second control point between middle right & lower right. */
#define WF_LOWER_RIGHT 					18	/* Lower right of warp frame. */

#define WF_LOWER_RIGHT_LOWER_MID_BCP1 	19	/* First control point between lower left & middle left. */
#define WF_LOWER_RIGHT_LOWER_MID_BCP2 	20	/* Second control point between lower left & middle left. */
#define WF_LOWER_MID 					21	/* Anchor point between lower left & upper left. */
#define WF_LOWER_MID_LOWER_LEFT_BCP1 	22	/* First control point between lower left & middle left. */
#define WF_LOWER_MID_LOWER_LEFT_BCP2 	23	/* Second control point between lower left & middle left. */

/* std menu categories */
enum {
	FHX_CATEGORY_CLEANUP,
	FHX_CATEGORY_COLORS,
	FHX_CATEGORY_CREATE,
	FHX_CATEGORY_DELETE,
	FHX_CATEGORY_DISTORT,
	FHX_CATEGORY_INPUT,
	FHX_CATEGORY_CALIBRATION,
	FHX_CATEGORY_OTHER,
	FHX_CATEGORY_PATHOPS,
	FHX_CATEGORY_SELECT,
	FHX_CATEGORY_TEXT,
	FHX_CATEGORY_TIFF,
	__end__
};

/* --------------------------------------- Writer options */
/*
	Note:
		Make sure these stay in sync with the corresponding flags
	in rdwr.h. (hds)
*/

#define	EXPORTFLAG_DOAREA			0x0001	/* export one page -- the area */
#define EXPORTFLAG_PAGEBOUNDARIES	0x0002	/* print page boundaries */

/* output flags */
#define WRITERFLAG_CANDOPREFS		0x0001	/* has a prefs dlg */
#define WRITERFLAG_CANDOPAGES		0x0002	/* can do page ranges */
#define WRITERFLAG_FILESONLY		0x0004	/* can only write to disk files */
#define WRITERFLAG_CANDOSELECTED	0x0008	/* can output selected items */
#define WRITERFLAG_ALL_IN_ONE		0x0010	/* output all as one doc not one doc/page */
#define WRITERFLAG_CANGETPAGERANGE	0x0020	/* can return/set page range */
#define WRITERFLAG_OPENFILEMANUALLY	0x0040	/* Let the Xtra open/close/create the files */

/* input flags */
#define WRITERFLAG_READFLAGS		0x8000	/* Read the flags coming in */

/* If the WRITERFLAG_GETSETPAGERANGE flag is set then use the flags below.
Note: Some bits can be used twice. This flag overrides the meaning of all bits.
*/
#define WRITERFLAG_GETSETPAGERANGE	0x4000
#define WRITERFLAG_SETPAGERANGE		0x8000	/* Get page if bit not set (double use) */
#define WRITERFLAG_STARTPAGE		0x2000	/* If not true then end page */
#define WRITERFLAG_GETSETPAGEMASK	0x1FFF	/* This is the page being returned */

/* ------------------------------------- Unit codes */

enum {
	UNITCODE_XPOS,
	UNITCODE_YPOS,
	UNITCODE_DIST
};

/* Color types */
#define COLORFLAG_EDITABLE 	0x00000001	/* color is editable, i.e., is not "Black", "White", etc. */
#define COLORFLAG_SPOT 		0x00000002	/* color is spot */
#define COLORFLAG_PROCESS 	0x00000004	/* color is four-color process */
#define COLORFLAG_TINT 		0x00000008	/* color is a tint of another color */
#define COLORFLAG_NEW 		0x00000010	/* color is new (not yet added) */

/* Part types */
enum {
	PART_TYPE_INVALID,
	PART_TYPE_NONE,
	PART_TYPE_HANDLE,
	PART_TYPE_PATH_HANDLE,
	PART_TYPE_SEGMENT,
	PART_TYPE_BASELINE,
	PART_TYPE_CHARACTER
};

/* Element types */
enum {
	ELEM_TYPE_INVALID,
	ELEM_TYPE_PATH,
	ELEM_TYPE_RECTANGLE,
	ELEM_TYPE_OVAL,
	ELEM_TYPE_TEXT_COLUMN,
	ELEM_TYPE_TEXT_IN_PATH,
	ELEM_TYPE_TEXT_ON_PATH,
	ELEM_TYPE_GROUP,
	ELEM_TYPE_BLEND,
	ELEM_TYPE_reserved_1,
	ELEM_TYPE_COMPOSITEPATH,
	ELEM_TYPE_EPSIMPORT,
	ELEM_TYPE_IMAGEIMPORT,
	ELEM_TYPE_NEW_BLEND,
	ELEM_TYPE_CLIP_GROUP,
	ELEM_TYPE_ENVELOPE,
	ELEM_TYPE_SYMBOLINSTANCE,
	ELEM_TYPE_PAGE_ELEM,
	ELEM_TYPE_SWFIMPORT,
	ELEM_TYPE_EXTRUSION,
	ELEM_TYPE_POLYGON,
	ELEM_TYPE_CONNECTOR_LINE
};

/* Element flags returned/set by FHIElemGet/SetFlags */
#define ELEMFLAG_LOCKED 	0x00000001	/* element is locked. */
						/* NOTE: By FreeHand tradition, you can set the fill & stroke on a locked
							object, or edit the text.  However, you cannot change the location
							of the object.  THEREFORE: FHICmdReplaceCurrentElem, FHICmdDeleteElem, & 
							FHICmdReorderElemAt will fail on locked objects. */


/*
	import types.
*/

enum {
	IMPORT_TYPE_TIFF,
	IMPORT_TYPE_EPS
};

enum {
	FHI_TIFF_ONEISWHITE, 
	FHI_TIFF_ONEISBLACK, 
	FHI_TIFF_RGB, 
	FHI_TIFF_CMYK
};
#define TIFFINFOREC_VERSION		0

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	MoaLong		version;				/* set to zero */
	MoaFixed 	xPointSize;				/* tiff dimensions in points */
	MoaFixed 	yPointSize;				/* tiff dimensions in points */
	MoaLong		xPixels;				/* tiff dimensions in pixels */
	MoaLong		yPixels;				/* tiff dimensions in pixels */
	MoaFixed 	xScale;					/* tiff scale (ie, pixels / points) */
	MoaFixed 	yScale;					/* tiff scale (ie, pixels / points) */
	MoaUlong 	rowBytes;				/* how many bytes per row */
	MoaShort 	colorSpace;				/* one of the above enums */
	MoaShort 	samplesPerPixel;		/* samples per pixel -- includes possible alpha */
	MoaShort	bitsPerSample;			/* bits per component (sample) */
	MoaShort	numPlanes;				/* if planar, the number of planes (1 if chunky) */
	MoaBool		isPlanar;				/* is the tiff data planar (vs chunky) */
	MoaBool		hasAlpha;				/* does the tiff have an alpha channel */
	MoaBool		hasGrayMap;				/* if TRUE, graymap field is used */
	MoaBool		unused1;				/* unused (for now) */
	MoaByte		grayMap[256];			/* gray remapping */
} FHITiffInfoRec;

#ifdef powerc
#pragma options align=reset
#endif

/* Layer flags */
#define LAYERFLAG_LOCKED		0x00000001
#define LAYERFLAG_VISIBLE		0x00000002
#define LAYERFLAG_EMPTY			0x00000004
#define LAYERFLAG_GUIDES		0x00000008
#define LAYERFLAG_DIVIDER		0x00000010
#define LAYERFLAG_EXISTS		0x00000020
#define LAYERFLAG_BACKGROUND	0x00000040

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	FhFxPoint		initialPoint;			/* initial point,  */
	FhFxPoint		currentPoint;			/* current point,  */
	FhFxPoint		previousPoint;			/* previous point,  */
	MoaUlong	modifiers;				/* modifier keys */
	MoaFixed	pressure;				/* pressure (0.0.. 1.0) */
	MoaUlong	charCode;				/* character code (0x0000..0xFFFF)
										0xFFFFFFFF if none - new for FH8
										Do not use in FH7 */
	MoaUlong	trackFlags;				/* returned flags */

	MoaShort	clickCount;				/* mouse click count (ONLY VALID IN ToolBegin) */
} FHXToolTrackRec;

#ifdef powerc
#pragma options align=reset
#endif

#define FHX_TRACK_FLAGS_NONE			0x00000000L
#define FHX_TRACK_FLAGS_UPDATE			0x00000001L	// retun from tool

// These are the same on the Mac OS and Windows. Aren't we lucky1
#define FHX_CHAR_CODE_BS_KEY			0x00000008L
#define FHX_CHAR_CODE_TAB_KEY			0x00000009L
#define FHX_CHAR_CODE_CR_KEY			0x0000000DL

#define FHX_CHAR_CODE_ESC_KEY			0x0000001BL

#ifdef MACINTOSH
	#define FHX_DBCHAR_CODE_LEFT_ARROW		0x0000001CL
	#define FHX_DBCHAR_CODE_RIGHT_ARROW		0x0000001DL
	#define FHX_DBCHAR_CODE_UP_ARROW		0x0000001EL
	#define FHX_DBCHAR_CODE_DOWN_ARROW		0x0000001FL

	#define FHX_CHAR_CODE_LEFT_ARROW		0x0000001CL
	#define FHX_CHAR_CODE_RIGHT_ARROW		0x0000001DL
	#define FHX_CHAR_CODE_UP_ARROW			0x0000001EL
	#define FHX_CHAR_CODE_DOWN_ARROW		0x0000001FL
#endif
#ifdef WINDOWS
	#define FHX_DBCHAR_CODE_LEFT_ARROW		0x0000FEACL
	#define FHX_DBCHAR_CODE_RIGHT_ARROW		0x0000FEAEL
	#define FHX_DBCHAR_CODE_UP_ARROW		0x0000FEADL
	#define FHX_DBCHAR_CODE_DOWN_ARROW		0x0000FEAFL

	#define FHX_CHAR_CODE_LEFT_ARROW		0x000000ACL
	#define FHX_CHAR_CODE_RIGHT_ARROW		0x000000AEL
	#define FHX_CHAR_CODE_UP_ARROW			0x000000ADL
	#define FHX_CHAR_CODE_DOWN_ARROW		0x000000AFL
#endif

#define FHX_CHAR_CODE_DEL_KEY			0x0000007FL

// These key codes are different on the Mac OS and Windows. Aren't we silly!
#ifdef MACINTOSH
	#define FHX_CHAR_CODE_ENTER_KEY		0x00000003L
	#define FHX_CHAR_CODE_PAGE_UP		0x0000000BL
	#define FHX_CHAR_CODE_PAGE_DOWN		0x0000000CL
#endif
#ifdef WINDOWS
	#define FHX_CHAR_CODE_ENTER_KEY		0x0000000AL
#endif

/* flag bits for FHXToolOptions */
#define FHX_TOOL_OPT_AUTOSCROLL				0x00000001	/* tool wants autoscrolling */
#define FHX_TOOL_OPT_WANTS_AUTOGRID			0x00000002	/* tool wants points gridded */
#define FHX_TOOL_OPT_WANTS_PRESSURE			0x00000004	/* tool wants pressure info */
#define FHX_TOOL_OPT_WANTS_KEY_CODES		0x00000008	/* tool wants key code info */
#define FHX_TOOL_OPT_WANTS_ALT_DRAG_MODE	0x00000010	/* tool wants reposition while drag mode */
/* and possibly more to come. */

/* values in "modifiers". several may be set. */
#define FHX_TOOL_MOD_SHIFT				0x00000001
#define FHX_TOOL_MOD_CTRL				0x00000002
#define FHX_TOOL_MOD_OPTION				0x00000010
#define FHX_TOOL_MOD_COMMAND			0x00001000
#define FHX_TOOL_MOD_IS_KEY_DOWN		0x80000000

/* this is the default value that most tools want */
#define FHX_TOOL_DEFAULTS	(FHX_TOOL_OPT_AUTOSCROLL | FHX_TOOL_OPT_WANTS_AUTOGRID)

/* Special flag to the element query routines to indicate
	that we are querying the current element. */
#define CURRENT_ELEMENT ((ElemparamP)0xFFFFFFFF)

/* values for the flags in the Path Triple routines. */
#define PATH_POINTTYPE_CORNER	0x00000001
#define PATH_POINTTYPE_TANGENT	0x00000002
#define PATH_POINTTYPE_CURVE	0x00000004
#define PATH_AUTOCURVE			0x00000008
#define PATH_POINTSELECTED		0x00000010

/* values for the flags in FHIPathGet/SetFlags */
#define PATH_EOFILL				0x00000001	/* path is even/odd fill */
#define PATH_FILLOPENPATH		0x00000002	//	fill open path

/* values for the flags in FHIOSetFlags */
#define OUTLINE_EOFILL			0x00000001	/* path is even/odd fill */

#define DOCINFOREC_VERSION		0

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	MoaLong 	version;
	MoaShort	printerResolution;	/* in DPI */
	MoaBool 	docEverSaved;
	MoaBool 	docNeedsSave;
} DocInfoRec;

#ifdef powerc
#pragma options align=reset
#endif

#ifdef MACINTOSH
	#include <Files.h>
	typedef FSSpec SysFileSpec;
#else
	typedef struct {
		MoaChar pathname[512];
	} SysFileSpec;
#endif

#define MAX_CUSTOM_DATA	1024

enum {
	FHI_DRAWMODE_KEYLINE,
	FHI_DRAWMODE_PREVIEW_FASTER,
	FHI_DRAWMODE_PREVIEW_BETTER,
	FHI_DRAWMODE_PREVIEW_BETTER2
};

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	MoaLong			version;			/* version of this record (always set to zero) */
	PMoaVoid		systemContext;		/* the system thing we draw into -- e.g., GrafPort. */
	FhFxPoint			systemClipRectLo;
	FhFxPoint			systemClipRectHi;	/* clip to this rectangle of the systemContext.
											this rectangle is treated as though the systemContext
											always has 0,0 as its origin. */
	FHIMatrixRec	drawMtx;			/* matrix to apply when drawing elements 
											(set to unity matrix if not desired) */
	FhFxPoint			drawRectLo;
	FhFxPoint			drawRectHi;			/* rectangle of the document to draw in.
											only items intersecting this rectangle
											will be drawn.
											if clipToDrawRect is FALSE, this is
											ignored. */
	ElemparamP		elemP;				/* if not NULL, then draw just this
											element. */
	MoaFixed		greekTypeSize;		/* text smaller than this should be greeked */
	MoaLong			drawMode;			/* one of the above enums */
	MoaBool 		dispTextEffects;	/* if TRUE, display text effects (ignore the preference) */
	MoaBool			useDrawRect;		/* if TRUE, use drawRect. */
	MoaBool			onlySelectedItems;	/* if TRUE, only the currently selected
											items in the document will be drawn. */
} FHIDrawInfoRec;

#ifdef powerc
#pragma options align=reset
#endif

/* Image Data choices in Output Options */
enum {
	FHI_IMAGEDATA_ASCII,					/* ASCII encoding */
	FHI_IMAGEDATA_BINARY,					/* Binary data */
	FHI_IMAGEDATA_NONE,						/* NONE  (OPI commnets only) */
	FHI_IMAGEDATA_BINARY_CROSS_PLATFORM		/* Binary data (cross platform) */
};
#define FHI_DEFAULT_COLORSTEPS	0
#define FHI_MIN_COLORSTEPS		8
#define FHI_MAX_COLORSTEPS		255
#define FHI_DEFAULT_FLATNESS	0
#define FHI_MIN_FLATNESS		1
#define FHI_MAX_FLATNESS		100

/* Output options */
#define OUTPUT_OPTS_REC_VERSION	0

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	MoaLong			version;				/* set to OUTPUT_OPTS_REC_VERSION */
	MoaLong			maxColorSteps;
	MoaLong			flatness;
	MoaLong			imageData;				/* ASCII, binary, or none */
	MoaBool			includeInvisible;		/* if TRUE, Include Invisible Layers. */
	MoaBool			splitComplexPaths;		/* if TRUE, Split complex paths. */
	MoaBool			convertRGBtoCMYK;		/* if TRUE, convert RGB TIFFs to CMYK. */
	MoaBool			outputSelectedOnly;		/* if TRUE, export selected objects only */
	MoaBool			includeOPI;				/* if TRUE, include OPI comments */
} FHIOutputOptsRec;

#ifdef powerc
#pragma options align=reset
#endif

#define FHI_IMPORT_USE_PASTEBOARD	-1

/* Printing */
#define TILE_NONE		0
#define TILE_MANUAL		1
#define TILE_AUTOMATIC	2

#define PRINT_OPTS_REC_VERSION	0

#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	MoaLong			version;				/* set to PRINT_OPTS_REC_VERSION */
	MoaChar			settingsFileName[256];	/* Name of print setting file */
	MoaLong			maxColorSteps;			/* max color steps for grad/rad fills. */
	MoaLong			flatness;				/* current global flatness */
	MoaFixed		paperHeight;
	MoaFixed		paperWidth;
	MoaFixed		scale;					/* output scale factor */
	MoaFixed		spreadSize;				/* current spread size */
	MoaFixed		tileOverlap;			/* current tiling overlap */
	MoaLong			tileMode;				/* tiling mode for output, can be
												TILE_NONE, TILE_MANUAL, or TILE_AUTOMATIC */
	MoaChar			transferFunction[256];	/* transfer function name */
	MoaFixed		customScreen;			/* custom screen frequency */
	MoaChar			userPrep[256];			/* Name of userprep file */
	MoaLong			firstPage;				/* first page to print */
	MoaLong			lastPage;				/* last page to print */
	MoaLong			numCopies;				/* number of copies to print */
	SysFileSpec		PPDFile;				/* PPD file to use */
											/* NOTE: always check hasPPDFile to see if this 
												field had been set */

	MoaBool			setSettingsFileName;	/* When Printing, use new settings file specified in settingsFileName */
											/* NOTE:  all the options below will override
												the values from settingsFileName
												if their set... field is TRUE */
	MoaBool			alignMarks;				/* draw alignment marks */
	MoaBool			setAlignMarks;
	MoaBool			binaryTiff;				/* tiff images can use binary data */
	MoaBool			setBinaryTiff;
	MoaBool			setMaxColorSteps;
	MoaBool			cutLines;				/* draw cut lines */
	MoaBool			setCutLines;
	MoaBool			emulsionDown;			/* print emulsion down */
	MoaBool			setEmulsionDown;
	MoaBool			fitToPaper;				/* fit output on paper */
	MoaBool			setFitToPaper;
	MoaBool			setFlatness;
	MoaBool			includeTiffData;		/* if false, omit tiff data */
	MoaBool			setIncludeTiffData;
	MoaBool			includeInvisible;		/* if TRUE, Include Invisible Layers. */
	MoaBool			setIncludeInvisible;
	MoaBool			landscape;				/* paper in landscape orientation */
	MoaBool			setLandscape;
	MoaBool			negative;				/* print negative image */
	MoaBool			setNegative;
	MoaBool			setPaperHeight;
	MoaBool			setPaperWidth;
		/* 
			the width and height of the paper, according to the system
			print driver. note that this is physical dimensions,
			not imageable area.
			
		 */
	MoaBool			setScale;
	MoaBool			separations;			/* print separations (not composite) */
	MoaBool			setSeparations;
	MoaBool			sepNames;				/* print color separation names */
	MoaBool			setSepNames;
	MoaBool			splitComplex;			/* split complex paths */
	MoaBool			setSplitComplex;
	MoaBool			setSpreadSize;
	MoaBool			setTileOverlap;
	MoaBool			setTileMode;
	MoaBool			printTitle;				/* print filename and date */
	MoaBool			setPrintTitle;
	MoaBool			setTransferFunction;
	MoaBool			hasPPDFile;					/* if TRUE, a PPD has been found and the file info copied to PPDFile
												if FALSE, no PPD info has been copied into PPDFile */
	MoaBool			setPPDFile;
	MoaBool			usePPD;					/* if FALSE, don't use ppd even if we have one */
	MoaBool			setUsePPD;
	MoaBool			rgbTiff;				/* is it ok to output rgb tiff? */
	MoaBool			setrgbTiff;
	MoaBool			setCustomScreen;
	MoaBool			setUserPrep;
	MoaBool			includeOPI;				/* include OPI comments */
	MoaBool			setIncludeOPI;
	MoaBool			autoRotate;				/* Automatic paper orientation */
	MoaBool			setAutoRotate;
	MoaBool			printSelected;			/* if TRUE, only print selected objects */
	MoaBool			setPrintSelected;
	MoaBool			spotAsCMYK;				/* if TRUE, spot colors get printed as process colors */
	MoaBool			setSpotAsCMYK;
	MoaBool			overrideHalftones;		/* if TRUE, individual object halftones are overriden */
	MoaBool			setOverrideHalftones;
	MoaBool			createFonts;			/* create fonts for printer (currently only used in J-version) */
	MoaBool			setCreateFonts;
	MoaBool			transverse;
	MoaBool			setTransverse;
} FHIPrintOptsRec;

#ifdef powerc
#pragma options align=reset
#endif

/* Graphic Find and Replace constants */
enum {
	FHI_PROBLEM_OVERPRINT,
	FHI_PROBLEM_HALFTONES
};
enum {
	FHI_SCOPE_SELECTION,	/* traverse current selection only */
	FHI_SCOPE_PAGE,			/* traverse current page only */
	FHI_SCOPE_DOCUMENT		/* traverse current document */
};
#define FHI_SELECTION_SET		0	/* set current selection */
#define FHI_SELECTION_ADD		1	/* add to current selection */
#define FHI_SELECTION_REMOVE	2	/* remove from current selection */

#define FHI_MATCH_FILLS (0x01)
#define FHI_MATCH_STROKES (0x02)
#define FHI_MATCH_ALL (FHI_MATCH_FILLS|FHI_MATCH_STROKES)

enum {
	FHI_EFFECT_ANY,
	FHI_EFFECT_HIGHLIGHT,
	FHI_EFFECT_INLINE,
	FHI_EFFECT_SHADOW,
	FHI_EFFECT_STRIKE,
	FHI_EFFECT_UNDERLINE,
	FHI_EFFECT_ZOOM,
	FHI_EFFECT_GRAPHIC
};

#define EMPTY_PREFIX -1
#define NO_PREFIX 0
#define ADD_PREFIX 1
#define SUB_PREFIX 2
#define DIV_PREFIX 3
#define MUL_PREFIX 4
#define GT_PREFIX 5
#define GE_PREFIX 6
#define LT_PREFIX 7
#define LE_PREFIX 8

#define FHI_FILL_COLORS (0x01)
#define FHI_STROKE_COLORS (0x02)
#define FHI_ALL_COLORS (FHI_FILL_COLORS|FHI_STROKE_COLORS)

enum {
	FHI_REMOVE_USELESS,
	FHI_REMOVE_OVERPRINT,
	FHI_REMOVE_HALFTONES,
	FHI_REMOVE_CONTENTS
};

/* Graphic search and replace can also specify these fill/stroke types */
#define FHI_FILL_ANY		1000
#define FHI_FILL_CUSTOM		1001
#define FHI_FILL_TEXTURED	1002

#define FHI_STROKE_ANY			1000
#define FHI_STROKE_CUSTOM		1001

enum {
	FHI_GSR_BLEND_SETSTEPS,
	FHI_GSR_BLEND_SETRESOLUTION
};

#define FHI_FACE_ANY		-1

#define FHI_DONT_CHANGE_SIZE	-1

/* new color flags */
#define COLOR_FLAG_CMYK				0x00000001
#define COLOR_FLAG_RGB				0x00000002
#define COLOR_FLAG_LAB				0x00000004
#define COLOR_FLAG_HEX				0x00000008

/* the maximum length of the name of custom data passed 
to the DocCustomData routines */
#define MAX_CUSTOMDATA_NAMELENGTH	128

typedef struct {
	PMoaSystemFileSpec				pFileSpec;
	MoaLong							code;
	MoaLong							cmd;
	MoaLong							key;
} ShortcutsUIProcData, *PShortcutsUIProcData;

enum {
	SUI_CODE_NONE,
	SUI_CODE_EXECUTE_DIALOG,
	SUI_CODE_INIT_FROM_FILE,
	SUI_CODE_GET_COMMAND,
	SUI_CODE_FOUND
};

#ifdef MACINTOSH
#ifndef __MENUS__
	#include <Menus.h>
#endif

#define MET_NOTHING			0			/* can't happen */
#define MET_COMMAND			1			/* command menu element */
#define MET_TOGGLE			2			/* toggle menu element */
#define MET_ENUM			3			/* enumerated menu element */
#define MET_TRISTATE		4			/* unchecked, checked, or latent*/
#define MET_MENU			5			/* item represents a menu, not a real item */
#define MET_MENUHIDDEN		6			/* a menu not in menu bar, only for cmd keys */
#define MET_ENUM_NO_CHECK	7			/* enumerated menu element */
#define MET_DESKACC			0xFF		/* desk accessory element */

typedef struct {
	MoaLong			iNdx;
	MoaLong			iSubNdx;
	MoaLong			iCommand;
	MoaLong			iType;
	MoaLong			iCount;				// number of items or sub menu id
	MoaLong			_iMenuID;
	MoaLong			_iMenuNdx;
	MoaLong			_iMenuKey;
	MoaLong			_iMenuStyle;
	MenuHandle		_hMenu;
	MenuHandle		_hSubMenu;
} MMenuNdxRec, *PMMenuNdxRec;
#endif

// Flags sed in FHINewArcPath and FHIGetArcPoints

#define ARCPATH_FLAGS_NONE		0
#define ARCPATH_FLAGS_CHORD		1
#define ARCPATH_FLAGS_PIE		2

typedef struct {
	FhFxPoint			aPredBCP;
	FhFxPoint			aPoint;
	FhFxPoint			aSuccBCP;
	MoaLong			iFlags;		// not used currently;
} MutPathTriple, *PMutPathTriple;

// end Flags sed in FHINewArcPath and FHIGetArcPoints

// Please keep these the same as they are internally to FreeHand to simplify things
#define FHI_ARROW_FILLED	0x01 
#define FHI_ARROW_STROKED	0x02 
#define FHI_ARROW_NEWSCALE	0x04	/* all newObj arrows have this bit set */

enum {
	FHIcxidTransparent,
	FHIcxidMagnify,
	FHIcxidLighten, 
	FHIcxidDarken, 
	FHIcxidInvert,
	FHIcxidGrayscale
}; 
typedef MoaLong FHILensXformID;

	/* lens Flag bit masks */
#define FHI_LENS_FLAG_NONE			0x0000
#define FHI_LENS_FLAG_CENTERPOINT	0x0001
#define FHI_LENS_FLAG_OBJONLY 	 	0x0002
#define FHI_LENS_FLAG_SNAP			0x0004

typedef MoaError (*FHICreateTiffWriteRowProc)(PIMoaStream outStream, MoaLong row, PMoaVoid pParam);
typedef MoaError (*FHIStreamCreateTiffWriteRowProc)(StreamID outStream, MoaLong row, PMoaVoid pParam);

// Please keep these the same as they are internally to FreeHand to simplify things
enum {
	FHIStream_AccessReadOnly,
	FHIStream_AccessWriteOnly,
	FHIStream_AccessReadWrite
};
typedef MoaLong FHIStreamAccessParam;

enum {
 	FHIUndefinedType, 
 	FHIStrokeType, 
 	FHIFillType, 
 	FHIContentType
}; 
typedef MoaLong FHIProcedureType;

#define FHI_SNAPSHOT_TRANSPARENT_LENSES		0x00000001		
#define FHI_SNAPSHOT_MAGNIFY_LENSES			0x00000002
#define FHI_SNAPSHOT_LIGHTEN_LENSES			0x00000004
#define FHI_SNAPSHOT_DARKEN_LENSES			0x00000008
#define FHI_SNAPSHOT_GRAYSCALE_LENSES		0x00000010
#define FHI_SNAPSHOT_INVERT_LENSES			0x00000020
#define FHI_SNAPSHOT_ALL_LENSES				0x0000003F
#define FHI_SNAPSHOT_NON_ALPHA_LENSES		0x00000032
#define FHI_EXPAND_TO_FH10_FORMAT			0x00000040
#define FHI_EXPAND_CONTENT_INLINE			0x00000080
#define FHI_RASTERIZE_TO_CMYK_IMAGES		0x00000100
#define FHI_IGNORE_RASTER_EFFECTS			0x00000200
#define FHI_EXPAND_FOR_SWF					0x00000400
#define FHI_EXPAND_SWF_EPS_RASTER			0x00000800

#ifdef MACINTOSH
// Directory Identification values.
// Please keep this table identical to the table in the muttypes.h files in the XDK
typedef enum {
	FHI_DIR_COLORS				= 0,
	FHI_DIR_DICTIONARY,
#ifdef FEATURE_FONTSET
	FHI_DIR_FONTSET,
#endif // FEATURE_FONTSET	
	FHI_DIR_HELP,
	FHI_DIR_KEYBOARD_SETTINGS,
	FHI_DIR_MACROMEDIA_COMMON,
	FHI_DIR_PRINTER_SETTINGS,
	FHI_DIR_SETTINGS,
	FHI_DIR_USERPREP,
	FHI_DIR_XTRAS,			// this will give you the user XTra folder
	FHI_DIR_STYLES,
	FHI_DIR_TOOLBARS,
	FHI_DIR_FREEHAND,
	FHI_DIR_ENVELOPE_PRESETS,
	FHI_DIR_SYMBOLS,
	FHI_DIR_PALETTES,
	FHI_DIR_GRAPHIC_HOSE
} FHI_DIRECTORY_ID;
#endif // MACINTOSH

#ifdef WINDOWS
// Directory Identification values.
// Please keep these values the same as DIRECTORY_ID defined in pc\edge\directory.h
typedef enum {
	FHI_DIR_FREEHAND			= 0,
	FHI_DIR_XTRAS				= 1,
	FHI_DIR_LANGUAGE			= 2,
	FHI_DIR_COLORS				= 3,
	FHI_DIR_PPDS				= 4,
	FHI_DIR_USERPREP			= 5,
	FHI_DIR_SETTINGS			= 6,
	FHI_DIR_PRINTER_SETTINGS	= 7,
	FHI_DIR_KEYBOARD_SETTINGS	= 8,
	FHI_DIR_TOOLBARS			= 9,
	FHI_DIR_STYLES				= 10,
	FHI_ENVELOPE_PRESETS		= 11,
	FHI_SYMBOLS					= 12,
	FHI_DIR_PALETTES			= 13,
	FHI_DIR_USERDATA			= 18
} FHI_DIRECTORY_ID;
#endif // WINDOWS

typedef MoaBoolParam (*FHIEnumFilesProc)(ConstPMoaChar pszFileName, PMoaVoid pParam);

// 971219 Crop data for images that have been cropped by the FH8 crop tool
// Added to fix "bug" (missing functionality) #5283 in eyedropper tool (smp)
#ifdef powerc
#pragma options align=mac68k
#endif

typedef struct {
	MoaShort	sFrameWidth;	// complete image dimensions (pixels)
	MoaShort	sFrameHeight;
	MoaShort	sCropWidth;		// dimensions after crop (pixels)
	MoaShort	sCropHeight;
	MoaShort	sCropOffsetX;	// lower-left corner offset from complete image (pixels)
	MoaShort	sCropOffsetY;
} FHICropInfoRec;

#ifdef powerc
#pragma options align=reset
#endif

// 980223 Language identification values for IMoaFHVersionInfo::FHIAppLanguage
typedef enum {
	FHI_LANG_NONE = 0,			// A bogus value good for initializing variables
	FHI_LANG_FIRST = 1,			// Make sure this is always first and that it
								// Matches the first language ID.  (used for loops)
	FHI_LANG_ENGLISH = 1,
	FHI_LANG_FRENCH,
	FHI_LANG_GERMAN,
	FHI_LANG_ITALIAN,
	FHI_LANG_SPANISH,
	FHI_LANG_SWEDISH,
	FHI_LANG_PORTUGESE,
	FHI_LANG_JAPANESE,
	FHI_LANG_CHINESE,
	FHI_LANG_SIMP_CHINESE,
	FHI_LANG_KOREAN,
	FHI_LANG_HEBREW,
		
	FHI_LANG_END 				// Make sure this one is always last (used for loops)
} FHI_LANGUAGE_ID;

// 980223 Platform and Sub-platform ID values for IMoaFHVersionInfo::FHIOSVersion
typedef enum {
	FHI_PLAT_WINDOWS	= 0,
	FHI_PLAT_MAC		= 1
} FHI_PLATFORM_ID;

#ifdef WINDOWS
// keep these the same as OSVERSIONINFO.dwPlatformId defines in winbase.h
typedef enum {
	FHI_SUBPLAT_WIN32S			= VER_PLATFORM_WIN32s,
	FHI_SUBPLAT_WIN95			= VER_PLATFORM_WIN32_WINDOWS,
	FHI_SUBPLAT_WINNT			= VER_PLATFORM_WIN32_NT
} FHI_SUBPLATFORM_ID;
#endif

#ifdef MACINTOSH
typedef enum {
	FHI_SUBPLAT_68K		= 0,
	FHI_SUBPLAT_PPC		= 1
} FHI_SUBPLATFORM_ID;
#endif

// 980223 License type values for IMoaFHVersionInfo::FHISerialNumber
// keep these the same as SN_* values in pc\edge\serial.h or mac\edge\aboutbox.c
typedef enum {
	FHI_SN_UNDEFINED = 0,
	FHI_SN_SINGLE,
	FHI_SN_5USER,
	FHI_SN_10USER,
	FHI_SN_20USER,
	FHI_SN_50USER,
	FHI_SN_UNLIMITED,
	FHI_SN_DEMO,
	FHI_SN_NFR,
	FHI_SN_BETA,
	FHI_SN_RESERVED,
	FHI_SN_STUDENT
} FHI_SN_LICENSE_TYPE;

// --- Double byte defines.  Used for doublebyt eversion --------------------

#define kDbcsCharAttrsVersion		2
#define kDbcsParaAttrsVersion		2

#define DBCS_LINEBREAK_LEVEL1		0
#define DBCS_LINEBREAK_LEVEL2		1
#define DBCS_LINEBREAK_LEVEL3		2

#define DBCS_RULES_NONE 			0
#define DBCS_RULES_JAPANESE 		1

#define DBCS_ROTATE_NONE			0
#define DBCS_ROTATE_ROMAN			1
#define DBCS_ROTATE_TEXT			2

#define	DBCS_BRACKETS_FULLWIDTH		0
#define	DBCS_BRACKETS_HALFWIDTH		1


// --- typedefs ---------------------------------------------------------

#ifdef powerc
#pragma options align=mac68k
#endif

/* -------------------------------------------------------- FHIDbcsCharAttrsRec */
/*
	Character attributes for doublebyte text
*/
typedef struct {
	MoaUlong version;						// set to kDbcsCharAttrsVersion
	MoaLong charSet;
	MoaFixed vertScaling;
	MoaLong rotateText;
	MoaLong glyphType;
	MoaBool setVertScaling;
	MoaBool setRotateText;
	MoaBool setGlyphType;
} FHIDbcsCharAttrsRec;


#ifdef powerc
#pragma options align=reset
#endif

#ifdef powerc
#pragma options align=mac68k
#endif

/* -------------------------------------------------------- FHIDbcsParaAttrsRec */
/* 
	Paragraph attributes for doublebyte text
*/

typedef struct {
	MoaUlong version;						// set to kDbcsParaAttrsVersion
	RangeT extraCharSpacing;
	MoaLong lineBreakingRule;
	MoaLong ruleOption;
	MoaLong bracketWidth;
	MoaFixed jjSpacing;
	MoaFixed jrSpacing;
	MoaBool jjAuto;
	MoaBool jrAuto;
	MoaBool emIndent;
	MoaBool tsumeOption;
	MoaBool setExtraCharSpacing;
	MoaBool setLineBreakingRule;
	MoaBool setRuleOption;
	MoaBool setEmIndent;
	MoaBool setBracketWidth;
	MoaBool setJjAuto;
	MoaBool setJrAuto;
	MoaBool setJjSpacing;
	MoaBool setJrSpacing;
	MoaBool setTsumeOption;
} FHIDbcsParaAttrsRec;

// Flags for FHIBeginElemTraversalEx

enum
{
	TRAV_OPTS_SELECTED			= 0x00000001,
	TRAV_OPTS_INSIDE_GROUPS		= 0x00000002,
	TRAV_OPTS_INSIDE_ENVELOPES	= 0x00000004,
	TRAV_OPTS_INSIDE_SYMBOLS	= 0x00000008,
	TRAV_OPTS_WHOLE_DOCUMENT	= 0x00000010,
	TRAV_OPTS_NON_CONTEXT_OBJS	= 0x00000020
};

#ifdef powerc
#pragma options align=reset
#endif
#ifdef WINDOWS
	#pragma pack(pop)
#endif
// --- interfaces -------------------------------------------------------

#undef INTERFACE
#define INTERFACE IMoaFhDbcsCallback

DECLARE_MOA_INTERFACE_(IMoaFhDbcsCallback, IMoaUnknown)
{
	STD_IMOAUNKNOWN_METHODS

	STDMOAMETHOD_(void, FHIDbcsTextInitCharAttrs) (THIS_ FHIDbcsCharAttrsRec* attrsP) PURE;
	STDMOAMETHOD_(void, FHIDbcsTextInitParaAttrs) (THIS_ FHIDbcsParaAttrsRec* attrsP) PURE;

	STDMOAMETHOD_(MoaLong, FHIDbcsTextGetCharAttrs) (THIS_ ElemparamP elem, MoaLong position, FHIDbcsCharAttrsRec* attrsP) PURE;
	STDMOAMETHOD_(MoaLong, FHIDbcsTextGetParaAttrs) (THIS_ ElemparamP elem, MoaLong position, FHIDbcsParaAttrsRec* attrsP) PURE;

	STDMOAMETHOD_(MoaBoolParam, FHIDbcsTextSetCharAttrs) (THIS_ ElemparamP elem, MoaLong position, MoaLong numChars, FHIDbcsCharAttrsRec* attrsP) PURE;
	STDMOAMETHOD_(MoaBoolParam, FHIDbcsTextSetParaAttrs) (THIS_ ElemparamP elem, MoaLong position, MoaLong numChars, FHIDbcsParaAttrsRec* attrsP) PURE;
};
typedef IMoaFhDbcsCallback FAR * PIMoaFhDbcsCallback;

// IMoaFhDbcsCallback: 44D3F31CD5B811CF98C600A0400E4CD8
MOA_DEFINE_GUID(IID_IMoaFhDbcsCallback, 0x44D3F31CL, 0xD5B8, 0x11CF, 0x98, 0xC6, 0x00, 0xA0, 0x40, 0x0E, 0x4C, 0xD8);

#endif	/* _H_muttypes */
