/* 
	file: pixelmoa.h

	interface descriptions for IMoaPixel... interfaces

	Revision History:
	960506	Added FhFixed-point resolution fields to the end of
			MoaImageInfoRec and incremented kVersioMoaImageInfoRec
			to 2 (jsd)
	950614	Now use "moaxtra.h" (jsd)
	950413	Original (jsd)
*/

#ifndef _H_pixelmoa
#define _H_pixelmoa

#ifndef _H_moaxtra
	#include "moaxtra.h"
#endif
#ifndef _H_moastdif
	#include "moastdif.h"
#endif

// ------------------------------------------------------- typedefs & so forth

#define kVersionMoaImageInfoRec					2

enum {
	MOA_IMAGE_ONEISWHITE,
	MOA_IMAGE_ONEISBLACK,
	MOA_IMAGE_INDEXEDCOLOR,
	MOA_IMAGE_RGB,
	MOA_IMAGE_CMYK
};

/* the MoaImageInfoRec is filled in with information about a pixel image.
The structure's alignment is the native alignment of the platform - ie
			MSVC default on win16 & win32
			mac68k on 68k
			power on powerc
*/
typedef struct {
	// a version field - set this to kVersionMoaImageInfoRec before using
	MoaUlong version;
	
	// image dimensions in pixels
	MoaUlong xPixels;
	MoaUlong yPixels;
	
	// image resolution in pixels per inch
	MoaUlong xRes;
	MoaUlong yRes;
	
	// info about the image space - this will be one of the above values
	MoaShort colorSpace;
	
	// info about the samples
	MoaShort bitsPerSample;
	MoaShort samplesPerPixel;
	MoaBool hasAlpha;
	MoaBool hasGraymap;
	MoaByte grayMap[256];

	// fields which are new to version 2 of the structure

	/* the following fields allow for more accurate representation
	of the image's resolution in pixels per inch. Filling them in
	is not required, but if they're not going to be used each should
	be set to zero. In any event, you must still fill in the xRes and
	yRes fields with rounded-off resolution values. */
	MoaFixed fixedXRes;
	MoaFixed fixedYRes;
} MoaImageInfoRec;

typedef MoaImageInfoRec FAR *MoaImageInfoRecPtr;

// ----------------------------------------------------------- IMoaPixelAccess

#undef INTERFACE
#define INTERFACE IMoaPixelAccess

DECLARE_MOA_INTERFACE_(IMoaPixelAccess, IMoaUnknown)
{
	STD_IMOAUNKNOWN_METHODS

	// methods specific to IMoaPixelAccess interface
	STDMOAMETHOD(BeginAccess) (THIS_ MoaByte FAR *foreColorP, MoaByte FAR *backColorP) PURE;
	STDMOAMETHOD(EndAccess) (THIS) PURE;
	STDMOAMETHOD(GetPixels) (THIS_ MoaLong left, MoaLong top, 
						  MoaLong right, MoaLong bottom,
						  MoaUlong loComp, MoaUlong hiComp,
						  MoaUlong FAR *pRowBytes, MoaHandle FAR *pImageDataH) PURE;
	STDMOAMETHOD(SetPixels) (THIS_ MoaHandle pixelDataH,
						  MoaLong left, MoaLong top, MoaLong right, MoaLong bottom,
						  MoaUlong loComp, MoaUlong hiComp,
						  MoaUlong rowBytes) PURE;
	STDMOAMETHOD(GetImageInfo) (THIS_ MoaImageInfoRecPtr pMoaImageInfo) PURE;
	STDMOAMETHOD(GetColorTable) (THIS_ MoaByte FAR *redTableP, 
							  MoaByte FAR *greenTableP, MoaByte FAR *blueTableP) PURE;
};

typedef IMoaPixelAccess FAR * PIMoaPixelAccess;

MOA_DEFINE_GUID(IID_IMoaPixelAccess, 0xABB2D7B2L, 0x000B, 0x5DEF, 0x00, 0x00, 0x08, 0x00, 0x07, 0x14, 0x7E, 0xD2);

// ------------------------------------------------------------- IMoaPixelMask

#undef INTERFACE
#define INTERFACE IMoaPixelMask

DECLARE_MOA_INTERFACE_(IMoaPixelMask, IMoaUnknown)
{
	STD_IMOAUNKNOWN_METHODS

	// methods specific to IMoaPixelMask interface
	STDMOAMETHOD(GetMask) (THIS_ MoaLong left, MoaLong top,
						MoaLong right, MoaLong bottom,
						MoaHandle FAR *pMaskDataH) PURE;	
};

typedef IMoaPixelMask FAR * PIMoaPixelMask;

MOA_DEFINE_GUID(IID_IMoaPixelMask, 0xABB2D7F2L, 0x000B, 0x6D2B, 0x00, 0x00, 0x08, 0x00, 0x07, 0x14, 0x7E, 0xD2);

#endif
