/*
	file: pixfltif.h (core)

	Macromedia Incorporated Proprietary Information - copyright 1995.
	For internal use only. Disclosure or transfer to unauthorized 
	persons prohibited. This computer software is protected under 
	copyright law as an unpublished work. All rights reserved.

	Abstract: This file identifies interfaces for pixel filter Xtras to
	use in Macromedia apps. PixelFilter Xtras are Xtras which modify
	raster images (or portions thereof) a la Photoshop filters.

	The three interfaces identified are:
	> IMoaPixelFilter - used for running a pixel filter against a 
	specified image
	> IMoaPixelFilterParms - used to get information about the arguments
	to a filter, and to set new argument values - in short, to allow
	the host application to modify a filter's settings

 	Revision history:
 	950809	Revised for MOA 1.3. (srj)
 	950721	Added the method IsFilterMoaNative to the IMoaPixelFilterInfo
 			interface. This method can be used to determine if the filter
 			is written to our API or is provided thru an adapter (as in
 			the Mastermind Xtra) if the host wishes to do something
 			different with the filter (eg, display it in a different
 			menu or what have you) (jsd)
	950608	Changed prototype of IMoaPixelFilterParms::GetCurrentParms
			to return a MoaError and set a MoaHandle given a pointer to
			it; also, added the error code kMoaErr_PixFiltParmsFormatUnknown
			(jsd)
	950606	Changed prototype of IMoaPixelFilterParms::CanTween() to return
			a MoaError and set a MoaBoolParam given a pointer to it (jsd)
	950511	Added define for kMoaErr_PixFiltErrorShown (jsd)
	950322	Original (jsd)
*/

#ifndef _H_pixfltif
#define _H_pixfltif

#include "moastdif.h"

#ifndef _H_moaxtra
	#include "moaxtra.h"
#endif

/* ------------------------------------------------- defines, typedefs, etc */

// some error codes
// wrong color space
#define kMoaErr_PixFiltBadColorSpace			MAKE_MOAERR(22000)
// an error occurred, but the user has already been alerted thru a dialog or whatever
#define kMoaErr_PixFiltErrorShown				MAKE_MOAERR(22001)
// the format of the filter's parameters can't be deciphered
#define kMoaErr_PixFiltParmsFormatUnknown		MAKE_MOAERR(22002)

/* -------------------------------------------------------- IMoaPixelFilter */

#undef INTERFACE
#define INTERFACE IMoaPixelFilter

DECLARE_MOA_INTERFACE_(IMoaPixelFilter, IMoaUnknown)
{
	STD_IMOAUNKNOWN_METHODS

	// methods specific to IMoaPixelFilter interface
	STDMOAMETHOD(Prepare) (THIS) PURE; 
	STDMOAMETHOD(Start) (THIS_ PIMoaUnknown pImageUnknown) PURE;
	STDMOAMETHOD(Continue) (THIS_ PIMoaUnknown pImageUnknown, MoaBool FAR *pDone) PURE;
	STDMOAMETHOD(Finished) (THIS_ PIMoaUnknown pImageUnknown) PURE;
	STDMOAMETHOD(Unprepare) (THIS) PURE;
	STDMOAMETHOD(DoAboutBox) (THIS) PURE;
};

typedef IMoaPixelFilter FAR * PIMoaPixelFilter;

MOA_DEFINE_GUID(IID_IMoaPixelFilter, 0x525fa910, 0x649c, 0x11ce, 0x98, 0x64, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1);

/*
	public information in the cache:
*/

#define key_IMoaPixelFilter_Name		"IMoaPixelFilter_Name"			// C String
#define key_IMoaPixelFilter_Category	"IMoaPixelFilter_Category"		// C String
#define key_IMoaPixelFilter_IsMoaNative	"IMoaPixelFilter_IsMoaNative"	// MoaBool
#define key_IMoaPixelFilter_CanTween	"IMoaPixelFilter_CanTween"		// MoaBool

/* --------------------------------------------------- IMoaPixelFilterParms */

#undef INTERFACE
#define INTERFACE IMoaPixelFilterParms

DECLARE_MOA_INTERFACE_(IMoaPixelFilterParms, IMoaUnknown)
{
	STD_IMOAUNKNOWN_METHODS

	// methods specific to IMoaPixelFilterParms interface
	STDMOAMETHOD(PromptForParms) (THIS_ PIMoaUnknown pImageUnknown) PURE;
	STDMOAMETHOD(GetCurrentParms) (THIS_ MoaHandle FAR *pParmsH) PURE;
	STDMOAMETHOD(UseParms) (THIS_ MoaHandle parmsH) PURE;
	STDMOAMETHOD(UseTweened) (THIS_ MoaHandle beginParmsH, MoaHandle endParmsH, 
						   MoaUlong numer, MoaUlong denom);
};

typedef IMoaPixelFilterParms FAR * PIMoaPixelFilterParms;

MOA_DEFINE_GUID(IID_IMoaPixelFilterParms, 0xb025bcb0, 0x649c, 0x11ce, 0x98, 0x64, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1);

#endif

/* eof */
