/*
	File: uidefs.h	    (adaptive)

	Copyright (c) 1994 Altsys. All rights reserved.
*/

#pragma once

#ifndef _H_uidefs
#define _H_uidefs

#ifdef WINDOWS
#ifndef _WINDOWS_
	#include <windows.h>
#endif
#endif


/*
	modifier key declarations
 */
#define UIVK_OPTIONALT	0x0001
#define UIVK_SHIFT		0x0002
#define UIVK_CTRL		0x0004
#define UIVK_CMD		0x0008	/* Mac only */

/*
	Types of gadgets.
*/
#define UICTL_ENDGROUP		-1	/* this is a pseudo-type used as a resource compiler directive. */
#define UICTL_INVALID		0
#define UICTL_CHECKBOX		1
#define UICTL_PUSHBUTTON	2
#define UICTL_RADIO			3
#define UICTL_TRISTATE		4
#define UICTL_ICONBUTTON	5
#if 1	/* RST PostRC1Fix */
	#define UICTL_NONMOVEABLEMODAL 6
#else
	#define UICTL_unused1		6	/* this slot no longer used */
#endif
#define UICTL_HSCROLLBAR	7
#define UICTL_VSCROLLBAR	8
#define UICTL_SLIDER		9
#define UICTL_DIAL			10
#define UICTL_LISTBOX		11
#define UICTL_COMBOBOX		12
#define UICTL_EDITTEXT		13
#define UICTL_STATICTEXT	14
#define UICTL_RADIOGROUP	15
#define UICTL_CUSTOMGADGET	16
#define UICTL_MODAL			17
#define UICTL_MODELESS		18
#define UICTL_SUBGROUP		19
#define UICTL_POPUPMENU		20
#define UICTL_SCROLLGROUP	21
#define UICTL_CMDMENU		22
#define UICTL_DIVIDERTEXT	23
#define UICTL_MODALGROW		24
#define UICTL_MODELESS_VERT	25
#define UICTL_COMBOBOXSORT  26
#define UICTL_GROUPBOX		27
#define UICTL_MODELESSGROW	28
#define UICTL_NEWEDITTEXT	29
#define UICTL_TABPANEL		30 			// FH11 only. A Studio UI tabbed panel
#define UICTL_POPUPCOLORWELL	31
#define UICTL_HELPBUTTON	32

/*
	keys for Get/Set routines.
	user keys can be added as negative values.
	all positive ones reserved for us
*/

#define UIKEY_VALUE					0	/* could be just about any type */
#define UIKEY_BOUNDS				1	/* msgParam2 is a UIRect* */
#define UIKEY_POSITION				2
#define UIKEY_TITLE					3
#define UIKEY_ICON					4
#define UIKEY_MIN					5
#define UIKEY_MAX					6
#define UIKEY_SELECTION				7
#define UIKEY_NEWSELECTION			8
#define UIKEY_COUNT					9
#define UIKEY_unused1				10
#define UIKEY_unused2				11
#define UIKEY_unused3				12
#define UIKEY_unused4				13
#define UIKEY_DEFAULT				14 	/* bool: control is "default" control (OK button) */
#define UIKEY_CANCEL				15	/* bool: control is "cancel" control (Cancel/Stop button) */
#define UIKEY_GROWABLE				16	/* bool: window is growable, or, scrollgroup needs space for growbox */
#define UIKEY_TINYFONT				17	/* bool: item uses small font -- useful only on Macintosh */
#define UIKEY_GOT_HSCROLL			18	/* bool: scrollgroup has h scrollbar */
#define UIKEY_unused5				19
#define UIKEY_PAGE_DELTA			20	/* long: number of units for page delta (scrolling) */
#define UIKEY_SCROLLBOUNDS			21	/* rect: the bounds of the scrolling area (scrollgroups only) */
#define UIKEY_SCROLLTO				22	/* point: move this point to topleft (scrollgroups only) */
#define UIKEY_SCROLLGROUPCONTENT	23	/* rect: return the sub-part of the scrollgroup's bounds that
										represents the area in which the scrollgroup's contents are visible.
										setting this is equivalent to setting UIKEY_SCROLLBOUNDS with
										some careful offsets. this key is useful for allowing
										platform-independent dealing with scrollbar sizes and such... */
#define UIKEY_NEWWINBOUNDS			24	/* rect: you will receive a UI_GET_UIRECT msg after a window
										is resized. the requested resize bounds is
										in msgParam2. you may modify the UIRect pointed to in order to
										modify the new size of the window. */
#define UIKEY_VISIBLE				25	/* bool: is gadget visible or not */
#define UIKEY_ENABLE				26	/* bool: is gadget enabled or not */
#define UIKEY_INCREMENT_DELTA		27	/* long: number of units for paging on arrow keys
										for scrollbars and scrollgroups */
#define UIKEY_GOT_VSCROLL			28	/* bool: scrollgroup has v scrollbar */
#define UIKEY_SYS_WINDOW			29	/* long: return the system window that the gadget is in.
											WindowPtr for Macintosh, HWND for Windows. */
#define UIKEY_SYS_EVENT				30	/* long: return a pointer to the most recent event. This does
											not "filter" for events specific to the gadget in question;
											it makes sense only to call this in response to messages
											to your window. type is an EventRecord*. (Macintosh only) */
#define UIKEY_ZOOMRECT				31	/* the Macintosh zoom rect for a window */
#define UIKEY_TEXTCENTER			32	/* center align static text field. This responds only to UIKEY_SETLONG. */
#define	UIKEY_TEXTRIGHT				33	/* right align static text field. This responds only to UIKEY_SETLONG. */
#define UIKEY_TEXTLEFT				34	/* left align static text field. This responds only to UIKEY_SETLONG. */
#define UIKEY_ZOOMSTATE				35	/* Mac: if T, window is "zoomed". */
#define UIKEY_WANTSKEYFOCUS			36	/* bool: if T, this gadget is able to become
											the "focused" gadget that accepts keydowns
											(e.g., editable text gadgets). This key is primarily
											useful for custom gadgets that want to implement
											keydown handling. */
#define UIKEY_INVALIDRECT			37	/* rect: area to invalidate */
// Note: The tree gadget is currently Mac only
#define UIKEY_NEWSUBTREE			38	/* Create a new sub tree in a tree gadget */
#define UIKEY_ENDSUBTREE			39	/* End the current sub tree in a tree gadget */
#define	UIKEY_DELETE_SELECTION		40	/* Delete the current selection */
#define	UIKEY_CURRENT_SELECTION		41	/* What is the current selection? */
#define UIKEY_OPENTREE				42	/* Open the current sub tree */
#define UIKEY_DEPTH					43	/* Get/Set the indentation depth of the current node */
#define	UIKEY_DELETE				44	/* Delete the given string/index */
#define UIKEY_SYS_CONTROL			45	/* long: return the system control for the gadget. (Mac only) */
#define UIKEY_ALLOW_TEXTWRAP		46  /* Allows text fields to wrap. On OS X10.2 text fields are single line by default */

// Note: End tree gadget keys

/*
	data types -- these are useful primarily for macintosh resources.
*/
#define UIDATATYPE_BOOL		0
#define UIDATATYPE_LONG		1
#define UIDATATYPE_FIXED	2
#define UIDATATYPE_POINT	3
#define UIDATATYPE_RECT		4
#define UIDATATYPE_STRING	5

/*
	UIMessages

	Important! : To add new messages, place them BEFORE UI_LASTMSG
*/
#ifdef WINDOWS
	#define UI_FIRSTMSG		(WM_USER+1)
#else
	#define UI_FIRSTMSG		(1)
#endif
#define UI_INITIALIZE		(UI_FIRSTMSG+1)    	/* Sent to a gadget when it is first created */
#define UI_ACTIVATE			(UI_FIRSTMSG+2)    	/* Sent to tell a gadget it has been activated */
#define UI_DEACTIVATE		(UI_FIRSTMSG+3)    	/* Sent to tell a gadget it is about to deactivate*/
#define UI_unused1			(UI_FIRSTMSG+4)
#define UI_ENDMODAL			(UI_FIRSTMSG+5)		/* UIRunModal is terminating */
#define UI_BEGINMODELESS	(UI_FIRSTMSG+6)		/* UIRunModeless was called */
#define UI_BEGINMODAL		(UI_FIRSTMSG+7)		/* UIRunModal was called */
#define UI_IDLE				(UI_FIRSTMSG+8)    	/* Sent to tell a gadget to perform idle tasks */
#define UI_KEYDOWN			(UI_FIRSTMSG+9)		/* Sent when a key is pressed in the gadget */
#define UI_unused2			(UI_FIRSTMSG+10)
#define UI_MOUSEDBLCLK		(UI_FIRSTMSG+11)	/* Sent when user double clicks in a gadget */
#define UI_MOUSEDOWN		(UI_FIRSTMSG+12)	/* Sent when the mouse goes down in a gadget */
#define UI_MOUSEUP			(UI_FIRSTMSG+13)	/* Sent when the mouse goes up in a gadget */
#define UI_MOUSETRACK		(UI_FIRSTMSG+14)	/* Sent while mouse moves within gadget */
#define UI_MOVE				(UI_FIRSTMSG+15)	/* Sent after the user has moved a gadget */
#define UI_REDRAW			(UI_FIRSTMSG+16)	/* Sent when a gadget needs to be redrawn */
#define UI_RESIZE			(UI_FIRSTMSG+17)	/* Sent when a gadget has been resized */
#define UI_unused3			(UI_FIRSTMSG+18)
#define UI_DESTROY			(UI_FIRSTMSG+19)	/* Sent before a gadget is destroyed */
#define UI_GADGETCHANGED	(UI_FIRSTMSG+20)	/* Sent when one of a gadget's controls changes value */
#define UI_GADGETTRACK		(UI_FIRSTMSG+21)	/* Sent while user is manipulating the control */
#define UI_unused4			(UI_FIRSTMSG+22)
#define UI_CANCLOSE			(UI_FIRSTMSG+23)	/* Sent to inquire if gadget is willing to close */
#define UI_CLOSE			(UI_FIRSTMSG+24)	/* Sent to close a gadget's window */
#define UI_GET_BOOL			(UI_FIRSTMSG+25)	/* Get data as bool */
#define UI_GET_LONG			(UI_FIRSTMSG+26)	/* Get data as long integer */
#define UI_GET_FIXED		(UI_FIRSTMSG+27)	/* Get data as FhFixed point */
#define UI_GET_UIPOINT		(UI_FIRSTMSG+28)	/* Get data as UIPoint */
#define UI_GET_UIRECT		(UI_FIRSTMSG+29)	/* Get data as UIRect */
#define UI_GET_STRING		(UI_FIRSTMSG+30)	/* Get data as C String */
#define UI_SET_BOOL			(UI_FIRSTMSG+31)	/* Set data as bool */
#define UI_SET_LONG			(UI_FIRSTMSG+32)	/* Set data as long integer */
#define UI_SET_FIXED		(UI_FIRSTMSG+33)	/* Set data as FhFixed point */
#define UI_SET_UIPOINT		(UI_FIRSTMSG+34)	/* Set data as UIPoint */
#define UI_SET_UIRECT		(UI_FIRSTMSG+35)	/* Set data as UIRect */
#define UI_SET_STRING		(UI_FIRSTMSG+36)	/* Set data as C String */
#define UI_unused5			(UI_FIRSTMSG+37)
#define UI_NEW_PANELID		(UI_FIRSTMSG+38)	/* sent to Studio UI Xtra panels when the panel is created or destroyed */
#define UI_TEXT_ENTERED		(UI_FIRSTMSG+39)	/* return or enter hit in an edittext field */
#define UI_MOUSEENTERED		(UI_FIRSTMSG+40)	/* sent when mouse enters gadget */
#define UI_MOUSEEXITED		(UI_FIRSTMSG+41)	/* sent when mouse exits gadget */
#define UI_MOUSETRACKUP		(UI_FIRSTMSG+42)	/* sent when mouse moves inside gadget when mouse is up */
#define	UI_DLGTABKEY		(UI_FIRSTMSG+43)	/* Windows only (used internally) */
#define	UI_BACKTABSETYOU	(UI_FIRSTMSG+44)	/* Windows only (used internally) */
#define UI_GETKEYFOCUS		(UI_FIRSTMSG+45)	/* sent to a gadget when it gains the key focus.
													this will only happen if the gadget returns
													TRUE for UIKEY_WANTSKEYFOCUS. */
#define UI_LOSEKEYFOCUS		(UI_FIRSTMSG+46)	/* sent to a gadget when it loses the key focus.
													this will only happen if the gadget returns
													TRUE for UIKEY_WANTSKEYFOCUS. */
#define UI_SELECTALL		(UI_FIRSTMSG+47)	/* this is sent when a field gains
													key focus after being tabbed into
													(as opposed to being clicked in).
													if is sent after the UI_GETKEYFOCUS
													message. */
// Note : These new message do not work with all the old gadget types
#define	UI_TELL_CUR_SEL		(UI_FIRSTMSG+48)	/* Broadcast the current selection */
#define UI_CLEARLIST		(UI_FIRSTMSG+49)	/* Clear the listbox/popupmenu/combobox */
#define UI_ENDLIST			(UI_FIRSTMSG+50)	/* End adding to the listbox/popupmenu/combobox */
#define UI_DELETE_SELECTION	(UI_FIRSTMSG+51)	/* message to delete the current selection */
#define UI_INSERTSTRING		(UI_FIRSTMSG+52)	// Inserts a string into a list (combobox only right now)
#define UI_GETITEMSTRING	(UI_FIRSTMSG+53)
#define UI_GETITEMINDEX		(UI_FIRSTMSG+54)
#define UI_SET_MAXLENGTH	(UI_FIRSTMSG+55)	// sets the max chars a text box can hold
#define UI_LASTMSG			(UI_FIRSTMSG+55)

/*
	Begin all custom messages here, not at UI_LASTMSG.
*/
#define UI_FIRSTUSERMSG		(UI_FIRSTMSG+1000)

#ifdef WINDOWS
	/* custom resource identifiers */
	#define UI_DIALOG 		1		/* built in dialog resource type */
	#define UI_CUSTOMGADGET	2		/* custom ctrl resource type */
	#define UI_GADGETLIST 	3		/* custom resource gadget list type */
	#define UI_ICONBUTTON	4		/* icon button */
	#define UI_COMBOBOX		5		/* combo box */
	#define UI_LISTBOX		6		/* list box */
	#define UI_SCROLLGROUP	7		/* scroll view control */
	#define UI_DIAL			8		/* dial control */
	#define UI_SUBGROUP		9		/* sub group changes */
	#define UI_POPUPMENU	10		/* popup menu control */
	#define UI_RADIOGROUP	11		/* radio group control */

	#define UI_BOOL 	UIDATATYPE_BOOL
	#define UI_LONG 	UIDATATYPE_LONG
	#define UI_FIXED 	UIDATATYPE_FIXED
	#define UI_POINT	UIDATATYPE_POINT
	#define UI_RECT		UIDATATYPE_RECT
	#define UI_STRING	UIDATATYPE_STRING
	#define ui_bool 	UIDATATYPE_BOOL
	#define ui_long 	UIDATATYPE_LONG
	#define ui_fixed 	UIDATATYPE_FIXED
	#define ui_point	UIDATATYPE_POINT
	#define ui_rect		UIDATATYPE_RECT
	#define ui_string	UIDATATYPE_STRING
	
	#ifdef COMMA
	#undef COMMA
	#endif // COMMA
	#define COMMA				"\0,"
	#define RESOURCE_STOP		(UICTL_ENDGROUP - 1)
	#define STOP				RESOURCE_STOP,COMMA
	#define END_RESOURCE		(RESOURCE_STOP - 1)
	#define RESOURCE_LINK_TAG	(SUB_RESOURCE - 1)

#endif

/* error messages returned as status codes */
#define UI_FALSE			0
#define UI_TRUE 			1

#define UI_NO_ERR			0	/* Everything is cool */
#define UI_OK				1	/* "OK" was selected */
#define UI_CANCEL 			2	/* "Cancel" was selected */
#define UI_CHECKED			3	/* button is checked */
#define UI_UNCHECKED		4	/* button is not checked */
#define UI_NOSTATE			5	/* button is indeterminate (TRI_STATE) */

#define UI_INVALID_GADGET	-3	/* You passed me something bogus */
#define UI_DESTROY_FAILURE	-4	/* Destruction attempt failed */
#define UI_MSG_NOT_SENT		-5	/* UISendMessage was not completed */
#define UI_CLASS_EXISTS		-6	/* The UI class exists */
#define UI_CLASS_DNE		-7	/* UI class Does Not Exist */
#define UI_MEM_ERR			-8	/* Memory allocation error occurred */
#define UI_INVALID_TYPE		-9	/* invalid gadget type "control" */
#define UI_GEN_ERR			-10	/* general error message */
#define UI_INVALID_KEY		-11	/* key passed was not valid */
#define UI_NOT_FOUND		-12 /* child gadget was not found */
#define UI_CREATE_FAILURE	-13 /* CreateGadget (or LoadGadget) failed */

#endif
